#include "include/main.h"
#include "../../src/include/map.h"
#include "include/list.h"
#include "include/test_with_structs.h"
#include "include/utils.h"
#include <stdio.h>
#include <string.h>
#include <time.h>

#define PRINT_INFO() printf("collisions: %ld, errors: %d\n", map->collisions, map->errors)

void test_map_set()
{
  map_T* map = NEW_MAP();
  map_set(map, "hello", strdup("world"));

  ASSERT(1, "assert reached");
  PRINT_INFO();
}

void test_map_get()
{
  map_T* map = NEW_MAP();
  map_set(map, "hello", strdup("world"));
  map_bucket_T* item = map_get(map, "hello");

  ASSERT(item != 0, "item is not null");
  PRINT_INFO();
}

void test_map_multiple()
{
  const char* keys[] = { "contextually-based",
                         "bi-directional",
                         "access",
                         "concept",
                         "Fundamental",
                         "5th generation",
                         "Reactive",
                         "productivity",
                         "hybrid",
                         "impactful",
                         "discrete",
                         "User-friendly",
                         "scalable",
                         "Customer-focused",
                         "info-mediaries",
                         "homogeneous",
                         "foreground",
                         "focus group",
                         "intangible",
                         "mission-critical",
                         "paradigm",
                         "client-driven",
                         "orchestration",
                         "algorithm",
                         "needs-based",
                         "Implemented",
                         "Exclusive",
                         "asymmetric",
                         "Profit-focused",
                         "ability",
                         "strategy",
                         "portal",
                         "hierarchy",
                         "collaboration",
                         "core",
                         "Open-architected",
                         "Fully-configurable",
                         "moderator",
                         "transitional",
                         "incremental",
                         "Centralized",
                         "Optimized",
                         "Universal",
                         "Progressive",
                         "Polarised",
                         "composite",
                         "client-server",
                         "projection",
                         "full-range",
                         "data-warehouse",
                         "Cross-platform",
                         "zero defect",
                         "product",
                         "regional",
                         "superstructure",
                         "Streamlined",
                         "ability",
                         "Optional",
                         "alliance",
                         "service-desk",
                         "De-engineered",
                         "knowledge base",
                         "asymmetric",
                         "demand-driven",
                         "Integrated",
                         "object-oriented",
                         "policy",
                         "local area network",
                         "3rd generation",
                         "Multi-lateral",
                         "De-engineered",
                         "3rd generation",
                         "optimal",
                         "upward-trending",
                         "grid-enabled",
                         "installation",
                         "Virtual",
                         "client-driven",
                         "executive",
                         "systemic",
                         "Function-based",
                         "groupware",
                         "website",
                         "neural-net",
                         "Re-contextualized",
                         "Future-proofed",
                         "Monitored",
                         "structure",
                         "matrix",
                         "infrastructure",
                         "Streamlined" };

  const char* values[] = {
    "Ophioglossaceae",  "Cyperaceae",    "Chenopodiaceae",   "Poaceae",          "Convolvulaceae",
    "Nymphaeaceae",     "Moraceae",      "Scrophulariaceae", "Opegraphaceae",    "Fabaceae",
    "Rhamnaceae",       "Fabaceae",      "Rubiaceae",        "Fabaceae",         "Fabaceae",
    "Teloschistaceae",  "Cucurbitaceae", "Apocynaceae",      "Scrophulariaceae", "Pteridaceae",
    "Dicranaceae",      "Poaceae",       "Campanulaceae",    "Teloschistaceae",  "Ranunculaceae",
    "Rubiaceae",        "Fabaceae",      "Cyperaceae",       "Fabaceae",         "Roccellaceae",
    "Aristolochiaceae", "Acanthaceae",   "Cactaceae",        "Myrtaceae",        "Adoxaceae",
    "Cyperaceae",       "Apocynaceae",   "Chenopodiaceae",   "Euphorbiaceae",    "Fabaceae",
    "Salicaceae",       "Asteraceae",    "Apiaceae",         "Lecanoraceae",     "Rosaceae",
    "Vitaceae",         "Thymelaeaceae", "Primulaceae",      "Liliaceae",        "Scrophulariaceae",
    "Polygonaceae",     "Violaceae",     "Verbenaceae",      "Poaceae",          "Lamiaceae"
  };

  unsigned int nrkeys = sizeof(keys) / sizeof(keys[0]);
  unsigned int nrvalues = sizeof(values) / sizeof(values[0]);

  map_T* map = NEW_MAP();

  for (unsigned int i = 0; i < nrvalues; i++) {
    if (i > nrkeys - 1)
      break;
    const char* k = keys[i];
    const char* v = values[i];

    map_set(map, (char*)k, strdup(v));
  }

  for (unsigned int i = 0; i < nrvalues; i++) {
    if (i > nrkeys - 1)
      break;
    const char* k = keys[i];
    const char* v = values[i];

    map_bucket_T* item = map_get(map, (char*)k);

    if (!ASSERT(item != 0, "assert value in map.")) {
      return;
    }
    if (!ASSERT(strcmp((char*)item->value, v) == 0, "assert item has expected value.")) {
      return;
    }
  }

  PRINT_INFO();
}

void test_map_change_value()
{
  map_T* map = NEW_MAP();
  map_set(map, "name", strdup("ianertson"));
  map_set(map, "github", strdup("github.com/sebbekarlsson"));
  map_set(map, "age", strdup("23"));

  map_bucket_T* bucket = map_get(map, "name");
  if (!ASSERT(bucket != 0, "item exists."))
    return;
  ASSERT(strcmp(bucket->value, "ianertson") == 0, "value is correct");

  map_set(map, "name", "sebastian");

  bucket = map_get(map, "name");
  ASSERT(bucket != 0, "changed item exists.");
  ASSERT(strcmp(bucket->value, "sebastian") == 0, "value has changed");
}

void test_map_resize()
{
  const char* keys[] = { "bdast6e@pinterest.com",
                         "syoellj6@opera.com",
                         "lyegoshin2c@squarespace.com",
                         "hwalne8w@ycombinator.com",
                         "mpech2v@columbia.edu",
                         "fstonhardby@seattletimes.com",
                         "asimonsot@indiatimes.com",
                         "czorzoniaj@youtube.com",
                         "fravel7h@netlog.com",
                         "mwaythingr9@yandex.ru",
                         "gbeert82@exblog.jp",
                         "hkocherri@forbes.com",
                         "jdray7b@un.org",
                         "mashbrookia@washingtonpost.com",
                         "dfardonix@ted.com",
                         "denriquesdu@answers.com",
                         "hmacci5p@ted.com",
                         "ecoltart8d@chron.com",
                         "cwiltshawh0@bing.com",
                         "glacyie@guardian.co.uk",
                         "tjammetm6@youtu.be",
                         "jwindlessjz@icq.com",
                         "ggrasonnk@canalblog.com",
                         "sspearrn@sohu.com",
                         "klawsb6@wikimedia.org",
                         "pmcshirieqb@livejournal.com",
                         "spickton5a@constantcontact.com",
                         "lmugglp@canalblog.com",
                         "sseresq8@feedburner.com",
                         "komoylan27@berkeley.edu",
                         "gmeersiv@adobe.com",
                         "fbraneykt@ihg.com",
                         "dfabbag@buzzfeed.com",
                         "fdankkk@ifeng.com",
                         "ejakemannd@skyrock.com",
                         "ybonnor67@ucoz.com",
                         "ncranmerecs@sphinn.com",
                         "abluneed@unicef.org",
                         "nblameyee@abc.net.au",
                         "rskonegi@google.fr",
                         "qdrogandh@moonfruit.com",
                         "sruspinebl@cnbc.com",
                         "mpeasbyqp@feedburner.com",
                         "ipillingergj@meetup.com",
                         "amithun37@oakley.com",
                         "cdavley68@mac.com",
                         "gmarples7u@ameblo.jp",
                         "lmungin8c@npr.org",
                         "mboots7w@elegantthemes.com",
                         "ltirrellis@51.la",
                         "sdumphreyshn@cnn.com",
                         "abodicumma@mapy.cz",
                         "ccrichley1z@mac.com",
                         "lurionh7@zdnet.com",
                         "sfey73@google.it",
                         "enansoner@berkeley.edu",
                         "gocrevy8f@goo.ne.jp",
                         "kspreyiy@ifeng.com",
                         "ccarlozzi61@sina.com.cn",
                         "mheitonor@vinaora.com",
                         "cpopleebi@sciencedirect.com",
                         "hpocklingtona1@google.nl",
                         "mantyshev5m@cloudflare.com",
                         "tsegoe5j@booking.com",
                         "ekillgusjq@mtv.com",
                         "dblundanjw@ameblo.jp",
                         "ndalrymplein@answers.com",
                         "hbentinckbw@soup.io",
                         "phammerberg1a@businessinsider.com",
                         "emettrickecr@google.com.au",
                         "amilby4y@sbwire.com",
                         "cdomnineyay@vistaprint.com",
                         "lbruna7@instagram.com",
                         "spaolozzi51@sohu.com",
                         "cgrinyerp8@examiner.com",
                         "swoodroofc@yahoo.com",
                         "fnorthrop50@mozilla.org",
                         "cgodrichew@netvibes.com",
                         "abarzenoz@army.mil",
                         "nfashame3@artisteer.com",
                         "besplin2o@nps.gov",
                         "dmargremij@home.pl",
                         "gteanby9r@g.co",
                         "cmathewson2i@constantcontact.com",
                         "wboschmann2x@pagesperso-orange.fr",
                         "vdowson23@discuz.net",
                         "mdunmore8g@163.com",
                         "ggroddenb1@fda.gov",
                         "tlinnelle5@jimdo.com",
                         "mpennycuickk5@furl.net",
                         "lduinbletonm2@t.co",
                         "sbrightgc@mayoclinic.com",
                         "trosenblath7l@house.gov",
                         "rpelzky@sitemeter.com",
                         "kbenoixiw@hatena.ne.jp",
                         "jpeadpn@japanpost.jp",
                         "efenbyet@sciencedirect.com",
                         "abalharrielm@youtu.be",
                         "ecolesrr@cdc.gov",
                         "bcollinghamam@google.de",
                         "cmuttockf8@paginegialle.it",
                         "ascrivner1d@symantec.com",
                         "bsoutherdenbu@latimes.com",
                         "bderringtonir@fda.gov",
                         "cleethemdb@adobe.com",
                         "lschusterq1@mail.ru",
                         "fmillyardpc@i2i.jp",
                         "mbriffett4c@hhs.gov",
                         "fmclugishro@toplist.cz",
                         "sbridiegz@reverbnation.com",
                         "jklimp6q@samsung.com",
                         "sbaudoneif@vk.com",
                         "rdoucettedg@go.com",
                         "ysonierho@cdc.gov",
                         "cyirrellhg@digg.com",
                         "ldenerp1@technorati.com",
                         "cangerstein8o@oracle.com",
                         "jlawillefy@i2i.jp",
                         "amadnerc7@eepurl.com",
                         "rachromovkw@shinystat.com",
                         "wscardifeild76@chron.com",
                         "lmatschoss9q@freewebs.com",
                         "vmoringg1@canalblog.com",
                         "wdelfqd@mit.edu",
                         "bantoniadesqt@vistaprint.com",
                         "hhinckesmanq4@kickstarter.com",
                         "fpillmanoy@ezinearticles.com",
                         "kgoulbornbr@cnet.com",
                         "dmelliardh@mac.com",
                         "fricketes@who.int",
                         "mmiddle4q@ox.ac.uk",
                         "cellicer@cloudflare.com",
                         "sbraundte8@psu.edu",
                         "tboerdermanb4@dmoz.org",
                         "boglassanepe@yale.edu",
                         "afreemantleka@blogs.com",
                         "akeppie6y@163.com",
                         "ejannawayb3@twitpic.com",
                         "kcaulier63@rediff.com",
                         "jgoodbannede@nationalgeographic.com",
                         "kfifoot5l@latimes.com",
                         "nmunciefz@epa.gov",
                         "mknaggsjx@shop-pro.jp",
                         "wmaccoughen6i@symantec.com",
                         "wlanningk8@sakura.ne.jp",
                         "rbreitlingfi@yolasite.com",
                         "lvalentimpp@wikipedia.org",
                         "nhaggisre@cdc.gov",
                         "lhauchdt@hexun.com",
                         "hthoriusgk@bluehost.com",
                         "tweinenf@sbwire.com",
                         "jteligine@typepad.com",
                         "jmortelj5@pen.io",
                         "adundridgeo3@theatlantic.com",
                         "jbenedettini8a@nationalgeographic.com",
                         "bmatijevicl@google.com.hk",
                         "mdelahunty@behance.net",
                         "achantera8@msu.edu",
                         "gpoxtonh2@zdnet.com",
                         "tmelwallkn@shinystat.com",
                         "tspekee4@dailymotion.com",
                         "dramsbothamn5@tmall.com",
                         "gshoesmith2w@com.com",
                         "htrelevanop@feedburner.com",
                         "eselwayda@tumblr.com",
                         "hburkino5@walmart.com",
                         "lgraysmarkio@privacy.gov.au",
                         "dskinnbp@pbs.org",
                         "cparkesonbt@reference.com",
                         "aviles45@live.com",
                         "llassenm4@yellowbook.com",
                         "mcholtongw@msn.com",
                         "ksteane5t@wisc.edu",
                         "aaxonk4@engadget.com",
                         "cjewesdq@wufoo.com",
                         "dsimonichj9@amazon.com",
                         "amcgowranp0@guardian.co.uk",
                         "ksterry42@aboutads.info",
                         "fwarind8@friendfeed.com",
                         "ncaskie7o@indiegogo.com",
                         "adelgadonz@mac.com",
                         "cdeioh@washington.edu",
                         "fballaam1q@purevolume.com",
                         "lpethybridge8s@patch.com",
                         "lstaker9d@360.cn",
                         "apatilloiq@homestead.com",
                         "bstickens28@seesaa.net",
                         "ccestardj2@sourceforge.net",
                         "sdewintonkm@mit.edu",
                         "greyburn4x@xrea.com",
                         "cjarrel3v@youtube.com",
                         "lallrightax@vk.com",
                         "kmashamar@slashdot.org",
                         "epreefd@rambler.ru",
                         "tpratore9@desdev.cn",
                         "jdominec92@princeton.edu",
                         "wmcwilliamscb@goo.gl",
                         "khonnicottrk@ovh.net",
                         "jlogue4i@uol.com.br",
                         "rarmatidgemw@creativecommons.org",
                         "ttaskerah@house.gov",
                         "bogersp6@blogspot.com",
                         "aarcher88@yellowbook.com",
                         "caverillm@oaic.gov.au",
                         "evasyukhnovi1@google.ru",
                         "cpinockn7@barnesandnoble.com",
                         "rdampct@dailymotion.com",
                         "jdosdelljg@freewebs.com",
                         "rjurczykmx@is.gd",
                         "nphaupfe@netscape.com",
                         "eharkinsof@whitehouse.gov",
                         "kdavidzonbx@bloglines.com",
                         "mbasire8i@desdev.cn",
                         "ccowlini2@bandcamp.com",
                         "aherreranoo@skype.com",
                         "inend31@dmoz.org",
                         "mcreadyqv@ifeng.com",
                         "ttaggertyae@wikispaces.com",
                         "mcinellijp@cnn.com",
                         "kmquhargepw@npr.org",
                         "emalamore1j@cisco.com",
                         "afilanrj@creativecommons.org",
                         "ccompson75@newsvine.com",
                         "cchristofidesnl@stanford.edu",
                         "lriddingq7@columbia.edu",
                         "jcirlos3b@ezinearticles.com",
                         "rdorney64@nsw.gov.au",
                         "ccrackel69@miibeian.gov.cn",
                         "pbuss86@paginegialle.it",
                         "bdankovl1@flickr.com",
                         "bduigenane1@simplemachines.org",
                         "preedyhough4k@bluehost.com",
                         "cmolsonab@tinyurl.com",
                         "rdockwrayef@ed.gov",
                         "jbosnellfo@skyrock.com",
                         "ccartmillom@prlog.org",
                         "telliottpf@sfgate.com",
                         "rstartin6r@amazon.com",
                         "gnussiinm@stanford.edu",
                         "evasnevjl@cocolog-nifty.com",
                         "sconnow3s@is.gd",
                         "ndudingaf@blogs.com",
                         "psparshottme@liveinternet.ru",
                         "ilarcierkf@t.co",
                         "escholig3y@unblog.fr",
                         "bhoolaghank9@reuters.com",
                         "dhebblewhitecp@opensource.org",
                         "mallbon98@timesonline.co.uk",
                         "knencim5@abc.net.au",
                         "kgibbesonok@dion.ne.jp",
                         "jbernardotte3g@ameblo.jp",
                         "idorricott9z@youtu.be",
                         "tstonesdw@i2i.jp",
                         "bgraffhamji@etsy.com",
                         "tluckingsa6@bluehost.com",
                         "bcutteridge5u@time.com",
                         "kbromwichpi@independent.co.uk",
                         "tbrantlmp@clickbank.net",
                         "wgrecel7@squarespace.com",
                         "jblackalllk@scientificamerican.com",
                         "sineson33@jiathis.com",
                         "tterbruggend6@nasa.gov",
                         "cgarrowayf1@java.com",
                         "mgyerpu@huffingtonpost.com",
                         "kmckmurriegq@bigcartel.com",
                         "mroscamps7x@flavors.me",
                         "mbasekep5@samsung.com",
                         "gpowrieqx@mapquest.com",
                         "rlazenbycz@wiley.com",
                         "romeara62@i2i.jp",
                         "pflipekp4@google.ru",
                         "bcolbert43@wikispaces.com",
                         "pvautrey5z@ebay.co.uk",
                         "tkislingburyd2@acquirethisname.com",
                         "etunneymo@tmall.com",
                         "mgalliardfv@360.cn",
                         "msadgroveb7@macromedia.com",
                         "ehelixmu@spotify.com",
                         "mleathard3j@histats.com",
                         "npieterlf@google.de",
                         "rsmale3m@disqus.com",
                         "hcapeling4e@woothemes.com",
                         "chincksjf@so-net.ne.jp",
                         "rstilgoekq@huffingtonpost.com",
                         "hpodbury3e@privacy.gov.au",
                         "escapensca@adobe.com",
                         "kkenealyi4@vinaora.com",
                         "mderricoatms@earthlink.net",
                         "rtipladyr2@homestead.com",
                         "ldownscw@blogspot.com",
                         "msimms10@aboutads.info",
                         "dwysome4h@phpbb.com",
                         "bburnyeaths@woothemes.com",
                         "dbennet4z@usda.gov",
                         "gwetherickig@so-net.ne.jp",
                         "croonyc9@hp.com",
                         "bwashbrook7k@examiner.com",
                         "stuft6o@seattletimes.com",
                         "sreppaportl5@cpanel.net",
                         "lgoricken0@nature.com",
                         "aocollopy2g@smh.com.au",
                         "eshellumkr@bloglovin.com",
                         "jgiloglylv@storify.com",
                         "fkuzemka49@cmu.edu",
                         "ebartell1k@goo.gl",
                         "nfrancklingu@google.co.uk",
                         "fjenkinsonle@scientificamerican.com",
                         "rblakesl0@alibaba.com",
                         "mgarment6h@joomla.org",
                         "cnendickai@goodreads.com",
                         "gphettisdd@netscape.com",
                         "echurmsgg@nbcnews.com",
                         "rhartgillrq@unesco.org",
                         "mpeasgood6n@vk.com",
                         "zbool2t@creativecommons.org",
                         "jisaacs6u@mail.ru",
                         "cskermanen@sogou.com",
                         "bwendover24@unicef.org",
                         "edreverb2@etsy.com",
                         "tbrissendenih@soup.io",
                         "vgianolohu@phpbb.com",
                         "marpino4f@linkedin.com",
                         "elanneyjc@walmart.com",
                         "bmcbrearty8x@google.com.hk",
                         "amarconi16@nhs.uk",
                         "gminshulli7@va.gov",
                         "askeermermn@php.net",
                         "ljeness3d@tripod.com",
                         "dcornforthr4@skype.com",
                         "gtreadgear4@netlog.com",
                         "jbogart5w@jugem.jp",
                         "lcultcheth6w@sun.com",
                         "abernolletad@github.com",
                         "yconen2h@netvibes.com",
                         "mbowdrey3l@noaa.gov",
                         "ematzeitispo@storify.com",
                         "dsidnall0@narod.ru",
                         "mschaumanngy@parallels.com",
                         "lamerici9v@sitemeter.com",
                         "gminton1r@nytimes.com",
                         "caudrittmi@smh.com.au",
                         "tcorsorod@theatlantic.com",
                         "rjephcotef0@dot.gov",
                         "gfinders3a@google.ru",
                         "emiramso2@berkeley.edu",
                         "jbrammerc4@mit.edu",
                         "tdegregoriop7@photobucket.com",
                         "kbrave58@ftc.gov",
                         "snowickf5@blogger.com",
                         "shetterich5r@vk.com",
                         "vstonerbj@people.com.cn",
                         "bmcwhorter3k@apple.com",
                         "llabincf@businessweek.com",
                         "idimondh5@paypal.com",
                         "pschowenburgh9@msu.edu",
                         "cjannequinqo@joomla.org",
                         "msemrad30@php.net",
                         "avasiliu9b@parallels.com",
                         "lfroomepb@sitemeter.com",
                         "dorringne@zimbio.com",
                         "smoulsdalekj@wiley.com",
                         "rmorrisonns@wsj.com",
                         "mcock6j@bandcamp.com",
                         "gpothecarylj@youtu.be",
                         "bmooringhk@sfgate.com",
                         "edrydaleit@goo.gl",
                         "cschottm7@aboutads.info",
                         "bhinstockjj@wix.com",
                         "lsheirlaw7z@boston.com",
                         "bmeriothqn@blogs.com",
                         "blanglandj1@free.fr",
                         "mgronouslr@xrea.com",
                         "ddechelle3r@google.fr",
                         "mdiggar57@google.co.uk",
                         "gdradeyei@gizmodo.com",
                         "bpiscopiellola@homestead.com",
                         "zrobkenhq@bloglines.com",
                         "awestpfel5n@sina.com.cn",
                         "cgoddardbk@uol.com.br",
                         "ddagwell93@infoseek.co.jp",
                         "moliviahz@dagondesign.com",
                         "nocklando9@cnet.com",
                         "fmckiddinh6@who.int",
                         "tguillartdm@irs.gov",
                         "rhayseyfu@webs.com",
                         "vkempeqc@plala.or.jp",
                         "lboote3t@hc360.com",
                         "codydr@storify.com",
                         "givshineg@telegraph.co.uk",
                         "rsykes96@java.com",
                         "dgrzegorzewicz6z@blinklist.com",
                         "bfrostd9@rakuten.co.jp",
                         "cdunlap54@mayoclinic.com",
                         "vsang2s@blogs.com",
                         "ghumphrey5e@dailymail.co.uk",
                         "mgerrett18@google.nl",
                         "vkynnd2j@yellowpages.com",
                         "ghemshalleq@nba.com",
                         "nmarrinerjb@guardian.co.uk",
                         "hturevillem1@cafepress.com",
                         "irickhuss66@shutterfly.com",
                         "rloaldayic@go.com",
                         "gsydallju@businesswire.com",
                         "lkellscq@alibaba.com",
                         "iedlerfr@accuweather.com",
                         "csiegequ@nbcnews.com",
                         "dbathersby4g@lulu.com",
                         "kbeldon8r@nytimes.com",
                         "hwisonkx@google.co.uk",
                         "adanielibf@ifeng.com",
                         "grhydderchnr@uiuc.edu",
                         "pmccreadyr6@xing.com",
                         "smacounk2@constantcontact.com",
                         "nroobottom95@dagondesign.com",
                         "mpoonebc@cnet.com",
                         "mgorbelll9@fda.gov",
                         "pramln@jugem.jp",
                         "opackingtonj0@unicef.org",
                         "hmcbethj8@desdev.cn",
                         "aprandygv@reverbnation.com",
                         "trosenkrantza2@amazonaws.com",
                         "lhakerhw@usnews.com",
                         "skettlesingqh@booking.com",
                         "vamber41@apache.org",
                         "ajasperfp@bravesites.com",
                         "wbeceril44@princeton.edu",
                         "tlewsley4n@nature.com",
                         "blavissrl@mail.ru",
                         "crosenfieldil@exblog.jp",
                         "emar8n@bloomberg.com",
                         "hllopisdi@cornell.edu",
                         "gforgan85@addthis.com",
                         "nfuzzeyl2@businessinsider.com",
                         "kkincaidmd@scientificamerican.com",
                         "ccalytond@usgs.gov",
                         "jrailton91@blogs.com",
                         "dlauranthv@nymag.com",
                         "bsherrardja@alexa.com",
                         "kpichanbo@home.pl",
                         "jmordykg@independent.co.uk",
                         "cballe4r@cnbc.com",
                         "ldigmanls@seattletimes.com",
                         "eslamakerba@google.com.au",
                         "kgodspeede2e@ustream.tv",
                         "izanettiii@yahoo.co.jp",
                         "rcoultardi3@aboutads.info",
                         "mbilovus5f@i2i.jp",
                         "fpleasauncebq@cbslocal.com",
                         "asargeauntpl@dion.ne.jp",
                         "jgligorijevic9o@walmart.com",
                         "dgottschalkqg@wordpress.com",
                         "ivanderwaltff@yellowbook.com",
                         "gdoey55@who.int",
                         "elittlecote5v@pcworld.com",
                         "kespinpg@deviantart.com",
                         "dsailrm@globo.com",
                         "zbaston3h@tmall.com",
                         "adils8t@cam.ac.uk",
                         "dworsfield9n@cloudflare.com",
                         "nwardesworth17@about.com",
                         "stinkhambn@altervista.org",
                         "jandreidj@is.gd",
                         "tsutterbyo0@hc360.com",
                         "hmehargp2@admin.ch",
                         "eburghalljd@slideshare.net",
                         "ymckearnen71@domainmarket.com",
                         "jdewick77@shinystat.com",
                         "ukirkbridelo@github.com",
                         "dmacgarveyje@cbsnews.com",
                         "dbettesworthk1@altervista.org",
                         "sdilletn4@vk.com",
                         "sstarte9x@netscape.com",
                         "rfoxleypy@ox.ac.uk",
                         "omilillocy@craigslist.org",
                         "hduggena0@mashable.com",
                         "bbednallmz@miitbeian.gov.cn",
                         "nbattams4p@microsoft.com",
                         "mbutterley2a@prweb.com",
                         "rprangleak@bloglovin.com",
                         "aworttny@prlog.org",
                         "sbalsellie3z@tmall.com",
                         "ckillingbackft@dion.ne.jp",
                         "bgrievson89@freewebs.com",
                         "msmeesbv@statcounter.com",
                         "asodoi5@exblog.jp",
                         "mtaitec@wordpress.org",
                         "vrasselllw@newsvine.com",
                         "vnicol74@ifeng.com",
                         "fstrongen2@infoseek.co.jp",
                         "jbootellg9@salon.com",
                         "aplunketgb@jigsy.com",
                         "tcicci4l@state.gov",
                         "dpinchinglc@craigslist.org",
                         "ahusk1u@google.de",
                         "bmundow9i@deviantart.com",
                         "hroustki@umich.edu",
                         "kbrandi79@joomla.org",
                         "wlilleycrop65@newsvine.com",
                         "chuotaw@squarespace.com",
                         "eguernerc6@baidu.com",
                         "dkerblerck@usnews.com",
                         "ehanalanav@facebook.com",
                         "cwykemq@nydailynews.com",
                         "rspeere3@scientificamerican.com",
                         "hdecristoforofx@vimeo.com",
                         "emckirtonhh@dailymotion.com",
                         "frallinpa@spiegel.de",
                         "glakesap@over-blog.com",
                         "zolleta3@abc.net.au",
                         "btrusse3n@nationalgeographic.com",
                         "tlowesgs@wikia.com",
                         "vcowelyc0@wix.com",
                         "demlyn7t@opensource.org",
                         "pfredianimy@cnbc.com",
                         "aandoverpm@dailymail.co.uk",
                         "tfarres2k@bbb.org",
                         "vkenyamdx@tripadvisor.com",
                         "ecollidge1x@jalbum.net",
                         "bwiggall3w@163.com",
                         "mduckerinoe@ow.ly",
                         "mlamartinegr@nba.com",
                         "hvallery9s@amazon.de",
                         "draiman5@theguardian.com",
                         "htassellds@wikia.com",
                         "mashpolep@loc.gov",
                         "wpeasey29@ycombinator.com",
                         "jdaulbyha@aol.com",
                         "ldeluca5o@rakuten.co.jp",
                         "egreenwood4b@kickstarter.com",
                         "fpluckj3@nifty.com",
                         "abraderl4@reverbnation.com",
                         "nschenkelmr@comsenz.com",
                         "szecchinio8@booking.com",
                         "kbiaggelli1h@w3.org",
                         "ijemisonx@washington.edu",
                         "dcopelli2u@businessweek.com",
                         "wjubblx@sitemeter.com",
                         "lcasaroli9k@csmonitor.com",
                         "rlawranson7f@reference.com",
                         "kalkerb@sina.com.cn",
                         "cmetzing7n@uiuc.edu",
                         "mgarred25@timesonline.co.uk",
                         "nreglara4@yellowbook.com",
                         "earnaldy53@bloomberg.com",
                         "kgwinn36@cornell.edu",
                         "bpennamanf@microsoft.com",
                         "csquirrell78@newyorker.com",
                         "cwyrek7@businesswire.com",
                         "rindruchn@oracle.com",
                         "bcarrellq0@hostgator.com",
                         "adaftorrg@blogspot.com",
                         "amccrossandl@army.mil",
                         "emartijnj@nasa.gov",
                         "cbadgersld@tmall.com",
                         "cduesbury5d@ucoz.ru",
                         "ewoolhouseep@people.com.cn",
                         "cjorczykjt@geocities.jp",
                         "bolfertmf@comsenz.com",
                         "kwellandc5@jugem.jp",
                         "gclaymj@dailymotion.com",
                         "gnormanvilleey@comcast.net",
                         "hthexton2n@wsj.com",
                         "hhoodlassjy@oaic.gov.au",
                         "dpicton2m@cmu.edu",
                         "kmilbankg4@parallels.com",
                         "acarrollhb@umn.edu",
                         "ddivine2y@archive.org",
                         "alunnqq@qq.com",
                         "dfruchond7@arstechnica.com",
                         "dcoviellog5@so-net.ne.jp",
                         "dchristophersgh@economist.com",
                         "ldyneleyqz@lycos.com",
                         "dgrowden7v@google.cn",
                         "ecolebecknt@ycombinator.com",
                         "rmccluskeyqs@wordpress.org",
                         "jlillegard4u@google.com",
                         "jcattermull5s@ustream.tv",
                         "vjindracekk6@illinois.edu",
                         "bregnardpq@squarespace.com",
                         "jschaffeldy@springer.com",
                         "bspohrmannjo@newsvine.com",
                         "akepling2b@bandcamp.com",
                         "mscollardg2@nymag.com",
                         "ybrideau97@earthlink.net",
                         "kpointing6s@statcounter.com",
                         "cgowland8b@youtu.be",
                         "sbendik15@umn.edu",
                         "mdimentgd@paypal.com",
                         "kwolfendell3u@economist.com",
                         "brisbie9f@multiply.com",
                         "jwealthallaa@alexa.com",
                         "csoutenhp@nationalgeographic.com",
                         "ihackforthi6@deliciousdays.com",
                         "jballinl6@livejournal.com",
                         "tdaltreya5@goo.ne.jp",
                         "mtempleton1c@businesswire.com",
                         "thallward7d@aol.com",
                         "scockertone6@google.pl",
                         "sbecraftmb@wikia.com",
                         "dtrumblee7@virginia.edu",
                         "dkettlestingkh@plala.or.jp",
                         "dchalkbz@yolasite.com",
                         "mmcquorkel6k@nih.gov",
                         "dmurfett2@livejournal.com",
                         "lerskin1b@cornell.edu",
                         "parntzenjk@e-recht24.de",
                         "rcaberapt@wunderground.com",
                         "cjakubovicz3i@mail.ru",
                         "kstefanovicel@samsung.com",
                         "bdyson7g@istockphoto.com",
                         "scowxa@upenn.edu",
                         "dgowman94@wikipedia.org",
                         "ngladtbachdo@sakura.ne.jp",
                         "smilsomgl@discuz.net",
                         "achild5i@forbes.com",
                         "sackensonnx@studiopress.com",
                         "bronchka12@discuz.net",
                         "dwison2r@printfriendly.com",
                         "slockneral@umn.edu",
                         "ilenahan6t@webnode.com",
                         "kayrtondv@forbes.com",
                         "sivattsoc@yandex.ru",
                         "slatehouse4w@ocn.ne.jp",
                         "smargerrison1m@dagondesign.com",
                         "mmingusrh@cbc.ca",
                         "dbeddofa@wiley.com",
                         "rshard2q@harvard.edu",
                         "jmanselleh@java.com",
                         "xcaillouhm@123-reg.co.uk",
                         "mbrise1f@latimes.com",
                         "gsayeshf@hugedomains.com",
                         "cdeckerm8@mapquest.com",
                         "camblerg8@nba.com",
                         "jkarim1i@nytimes.com",
                         "heveleighk0@webeden.co.uk",
                         "lneweynq@lycos.com",
                         "emounceycu@gravatar.com",
                         "pbirdfieldgn@house.gov",
                         "spourveer8z@blog.com",
                         "shuggettn8@mysql.com",
                         "mchithamqy@surveymonkey.com",
                         "ajacop2d@slideshare.net",
                         "vreubbensaq@usnews.com",
                         "pknowltoni8@geocities.jp",
                         "dbeaj7@hugedomains.com",
                         "lmolangm@so-net.ne.jp",
                         "wpykettk3@fda.gov",
                         "bbaynsi@csmonitor.com",
                         "tsowood56@wix.com",
                         "ltutin5b@java.com",
                         "lorneblowm9@wiley.com",
                         "gcrawshayh1@opensource.org",
                         "vchasmor7p@purevolume.com",
                         "rlarkby8v@1und1.de",
                         "lgreenset13@hao123.com",
                         "hvatcherbb@facebook.com",
                         "mszwandtej@china.com.cn",
                         "ktuiteik@alibaba.com",
                         "dgyngylln9@hp.com",
                         "ssprowsonnu@fc2.com",
                         "mgarnsworthyqj@dropbox.com",
                         "stwigg22@archive.org",
                         "rcaramuscia8y@princeton.edu",
                         "pbrandimm@indiegogo.com",
                         "ddawks7a@oakley.com",
                         "lrounds2p@virginia.edu",
                         "ewatson32@soup.io",
                         "bmicahjn@usnews.com",
                         "kbucktrout9u@delicious.com",
                         "mfuryov@imdb.com",
                         "lmcconnellc2@slashdot.org",
                         "rwhetsonrd@hp.com",
                         "rbrindedf7@imdb.com",
                         "skevishi@examiner.com",
                         "mwardhawkb@bloglines.com",
                         "llantiff3o@house.gov",
                         "aspurrettng@wix.com",
                         "jmacneishmt@wikipedia.org",
                         "odjurisicn3@google.pl",
                         "gyouell6a@cdbaby.com",
                         "jthirkettlefl@usatoday.com",
                         "seastbymc@springer.com",
                         "pdemetrqf@psu.edu",
                         "cdelgardob0@apple.com",
                         "hmaclisef2@bizjournals.com",
                         "ltwycross8m@apple.com",
                         "mbulfoyd0@photobucket.com",
                         "shyde20@yelp.com",
                         "sfaraganv@uiuc.edu",
                         "aforstallnw@aol.com",
                         "kgentzsch39@google.nl",
                         "tgarfitt83@stanford.edu",
                         "galderseyao@sakura.ne.jp",
                         "sstollencm@tumblr.com",
                         "iivermeedn@cbc.ca",
                         "wmacalpyneq2@abc.net.au",
                         "askillett6f@devhub.com",
                         "krusean@ehow.com",
                         "tmealiffeno@newsvine.com",
                         "tbriteec8@joomla.org",
                         "aganeph@blogger.com",
                         "mtodeogp@usgs.gov",
                         "jvallens84@theglobeandmail.com",
                         "asharman59@imgur.com",
                         "pmccollumow@ask.com",
                         "rcoade2f@naver.com",
                         "cluckings70@walmart.com",
                         "ahonackerdf@paginegialle.it",
                         "kburyfm@joomla.org",
                         "joliddyli@usnews.com",
                         "edeleekkd@ameblo.jp",
                         "mcollarj4@alexa.com",
                         "htuxselllu@ca.gov",
                         "jstickingslh@technorati.com",
                         "mghilardi81@storify.com",
                         "pleathard1w@wired.com",
                         "lfortinos@bizjournals.com",
                         "nhonniebaleu@auda.org.au",
                         "nlethbridgeq@dailymail.co.uk",
                         "jtowson4d@ustream.tv",
                         "cwrathmallo@cdc.gov",
                         "mbushnellc3@google.co.uk",
                         "wgullickhy@simplemachines.org",
                         "loxenford6b@whitehouse.gov",
                         "estourtonly@home.pl",
                         "tprottq3@360.cn",
                         "kmattionibg@flickr.com",
                         "kkingswoodcv@rediff.com",
                         "njanowski4j@umich.edu",
                         "shourstonmh@blogspot.com",
                         "mobeypj@amazon.de",
                         "gwigan9j@yellowbook.com",
                         "cbudnikrb@soundcloud.com",
                         "lbambercj@dion.ne.jp",
                         "ecaukillh3@usnews.com",
                         "amaingotib@hostgator.com",
                         "kjatczakcc@bloglovin.com",
                         "agrenedc@princeton.edu",
                         "vheish4@usnews.com",
                         "galdredg7@mapy.cz",
                         "wandrickga@ebay.co.uk",
                         "keardleyjr@spotify.com",
                         "jzealander5q@soundcloud.com",
                         "vkrolak40@intel.com",
                         "elermouth1g@wiley.com",
                         "nfreschidk@livejournal.com",
                         "calster7c@woothemes.com",
                         "mmowenpx@pinterest.com",
                         "gchongim@sciencedirect.com",
                         "aileyo1@biblegateway.com",
                         "adashkovra@e-recht24.de",
                         "ksteptoac@woothemes.com",
                         "lgilbertbh@utexas.edu",
                         "grihanekiu@mayoclinic.com",
                         "nkiltie9p@un.org",
                         "mveillardmk@wp.com",
                         "bmastertonfh@baidu.com",
                         "eceneyox@weebly.com",
                         "bwhissellk@umn.edu",
                         "dcollingridget@squidoo.com",
                         "asoal14@nifty.com",
                         "uwetheredd4@google.fr",
                         "wdunge8k@sitemeter.com",
                         "sgarmonjv@upenn.edu",
                         "rskenemg@disqus.com",
                         "bsammons9g@google.com.hk",
                         "liiannonikl@alexa.com",
                         "hscalerap3@themeforest.net",
                         "vrarityqe@oracle.com",
                         "cflawncg@php.net",
                         "esambrooklt@unesco.org",
                         "atindallco@discovery.com",
                         "hhackey4s@quantcast.com",
                         "gcobbe46@xing.com",
                         "gbunson3f@gov.uk",
                         "sgenteryf4@1688.com",
                         "ephizakarleyoj@nydailynews.com",
                         "zmaingot3q@meetup.com",
                         "kpattersa9@jigsy.com",
                         "dphysickf9@unc.edu",
                         "ogrossman21@google.com",
                         "ntawse9m@comsenz.com",
                         "evanleeuwen35@desdev.cn",
                         "glittqk@stumbleupon.com",
                         "lgenge6@skyrock.com",
                         "gkubatschog@biglobe.ne.jp",
                         "cfisbburne4m@discuz.net",
                         "dfirmage6v@mapquest.com",
                         "jabisettib5@cbslocal.com",
                         "lfawdriegf@mozilla.org",
                         "gcannonhr@ow.ly",
                         "fholseyku@google.cn",
                         "olaflinni@liveinternet.ru",
                         "csallengerqi@marketwatch.com",
                         "mwildenm0@wired.com",
                         "vbellayps@state.gov",
                         "mdudlestone5g@fc2.com",
                         "cmauntonq5@rediff.com",
                         "kscollandqm@ustream.tv",
                         "abrodbin1t@purevolume.com",
                         "csirette6m@skyrock.com",
                         "ddelmontikp@europa.eu",
                         "fstothartnc@reddit.com",
                         "vweightev@wordpress.com",
                         "fsigwardcl@behance.net",
                         "clufkin34@irs.gov",
                         "ahutley1e@opensource.org",
                         "cloynsch@fotki.com",
                         "kgallenkv@1und1.de",
                         "lsharplyl3@yellowbook.com",
                         "edrezzer52@soup.io",
                         "emonktong@phoca.cz",
                         "mokiei0@google.it",
                         "mblaiseez@ucla.edu",
                         "ilogesdalehe@wsj.com",
                         "kharkinsh8@unblog.fr",
                         "rhedworthfs@prweb.com",
                         "ijadonn@foxnews.com",
                         "dsanchizfq@irs.gov",
                         "jsouthernfk@elpais.com",
                         "mstrathdeehd@goo.gl",
                         "cbastianu@github.io",
                         "bjarley80@a8.net",
                         "brosensaftf6@washington.edu",
                         "gdeverick47@ameblo.jp",
                         "pfoalece@indiegogo.com",
                         "rprewer4t@odnoklassniki.ru",
                         "dingremn1@hhs.gov",
                         "rswatlandhc@youku.com",
                         "cgrindellci@cargocollective.com",
                         "froussellr5@pagesperso-orange.fr",
                         "bstandevenbm@chicagotribune.com",
                         "lwallbridgedp@unblog.fr",
                         "jpepye7i@gov.uk",
                         "wbatie9c@who.int",
                         "tlettucee0@github.io",
                         "vpadilla7q@bigcartel.com",
                         "dcreaghrf@netvibes.com",
                         "ccrackel72@amazonaws.com",
                         "mginiei9@intel.com",
                         "rdobbsem@wp.com",
                         "mhosbyfw@ustream.tv",
                         "wbowler8j@weather.com",
                         "ddunbletonpk@nih.gov",
                         "nstebbinml@deliciousdays.com",
                         "kbehrendsenq6@cocolog-nifty.com",
                         "jwixey7s@blogspot.com",
                         "ltolchargt@hubpages.com",
                         "sdoeylz@earthlink.net",
                         "danthonyd3@newyorker.com",
                         "mbootynj@woothemes.com",
                         "sverheydenr3@arstechnica.com",
                         "morwing3@columbia.edu",
                         "bdabels9y@wunderground.com",
                         "wupstonql@wordpress.com",
                         "egladwellll@goo.ne.jp",
                         "ccosker8p@usnews.com",
                         "vrenzullo1n@theglobeandmail.com",
                         "alowtherg0@mapquest.com",
                         "kstandleykz@nsw.gov.au",
                         "bkubczak6l@amazonaws.com",
                         "lredbourn3x@cbc.ca",
                         "bsergentcn@google.com.hk",
                         "jbeatiz@state.tx.us",
                         "rstampfer9t@creativecommons.org",
                         "dwhaley7e@zimbio.com",
                         "emegsonex@wunderground.com",
                         "stowne38@newyorker.com",
                         "fellisso7@geocities.com",
                         "mdelcastel8h@cdc.gov",
                         "rwegener8e@illinois.edu",
                         "kfrisbypv@woothemes.com",
                         "hcaltun7@independent.co.uk",
                         "edimblebyht@pcworld.com",
                         "lbenzab9@intel.com",
                         "eporkerna@seattletimes.com",
                         "bthaineip@gnu.org",
                         "jabrahamowitcz1@miitbeian.gov.cn",
                         "hbridgettkc@tiny.cc",
                         "dganterq9@home.pl",
                         "ejeffcoate1o@virginia.edu",
                         "zegginsek@pagesperso-orange.fr",
                         "iastillfg@deviantart.com",
                         "fjoutapaitisn6@csmonitor.com",
                         "gwoolfootjm@sbwire.com",
                         "dcocklin5c@storify.com",
                         "gseabrookee2@dyndns.org",
                         "epoyntergo@unc.edu",
                         "eocorhane7m@over-blog.com",
                         "fcrawshay9@go.com",
                         "cmaydwaycx@cbsnews.com",
                         "mfrandsen1s@cbsnews.com",
                         "hflewin8l@nsw.gov.au",
                         "mquan7j@soundcloud.com",
                         "kcorballyfc@hugedomains.com",
                         "gpookenp@dell.com",
                         "mheitonol@oaic.gov.au",
                         "lbartlemanoq@msn.com",
                         "lbenhamrc@webmd.com",
                         "arofes@samsung.com",
                         "kbaisepd@biblegateway.com",
                         "msaladino6g@imdb.com",
                         "scommanderlb@mayoclinic.com",
                         "pmityashevpz@opera.com",
                         "gdykasg6@cyberchimps.com",
                         "mfominov9h@wisc.edu",
                         "mofinanea@xrea.com",
                         "rpawling8@privacy.gov.au",
                         "lzannini5y@nps.gov",
                         "ckelliebd@blogspot.com",
                         "lheddeo6@intel.com",
                         "ahessp9@flickr.com",
                         "mnisco1l@wikia.com",
                         "lkeegan4a@un.org",
                         "cgirt1y@icio.us",
                         "msilmanhx@printfriendly.com",
                         "csterzakerr0@utexas.edu",
                         "jesby4v@desdev.cn",
                         "lrubbert9l@addthis.com",
                         "llowndesob@edublogs.org",
                         "ttanswell1p@google.com.au",
                         "dvanin19@fema.gov",
                         "alinec1@squarespace.com",
                         "lgarriockeb@g.co",
                         "kmacgeffen6d@dropbox.com",
                         "ihyner8@sourceforge.net",
                         "goverelko@desdev.cn",
                         "aeggle2z@illinois.edu",
                         "rfolomkin7r@gmpg.org",
                         "tnavein9e@acquirethisname.com",
                         "bfernelyd5@cnn.com",
                         "zdunlop8u@imageshack.us",
                         "bbelshawlg@cam.ac.uk",
                         "abrickellm3@shinystat.com",
                         "pstobbs9w@feedburner.com",
                         "efarensas@so-net.ne.jp",
                         "iravelusrp@omniture.com",
                         "htuddallhj@utexas.edu",
                         "nmacenzy6p@amazonaws.com",
                         "achristophersonaz@cam.ac.uk",
                         "dgomar5x@163.com",
                         "pfardo99@adobe.com",
                         "mclouston11@shutterfly.com",
                         "adelacote6c@biblegateway.com",
                         "dpalleske90@wordpress.com",
                         "mjuanes9a@tripod.com",
                         "mgwillymfj@thetimes.co.uk",
                         "nberickid@photobucket.com",
                         "cnassau1v@newyorker.com",
                         "cizardz@multiply.com",
                         "edahlgrenqr@ibm.com",
                         "tlatedz@princeton.edu",
                         "htabbitt60@mashable.com",
                         "ybyarscd@tripod.com",
                         "hvallintinefn@soundcloud.com",
                         "pwapplingtonqw@narod.ru",
                         "ikeasey4o@sun.com",
                         "lpopleyou@arstechnica.com",
                         "llinnejs@shop-pro.jp",
                         "gheiton87@fda.gov",
                         "sordelt3p@wufoo.com",
                         "idelacroix5k@nps.gov",
                         "mkitchingd1@marriott.com",
                         "bparisshl@addthis.com",
                         "bmccannyoi@nature.com",
                         "hvolagebs@printfriendly.com",
                         "thastqa@ustream.tv",
                         "dboldpr@psu.edu",
                         "rbelham5h@archive.org",
                         "rmencob8@illinois.edu",
                         "jmulberryf3@arizona.edu",
                         "dpflegero4@oakley.com",
                         "rbaileyfb@cloudflare.com",
                         "governh@qq.com",
                         "rsongist48@wufoo.com",
                         "cmargaretau@nsw.gov.au",
                         "sivain3c@aol.com",
                         "hlortzoa@economist.com",
                         "iprichardgx@tamu.edu",
                         "gluciusbe@acquirethisname.com",
                         "fboswardjh@myspace.com",
                         "ghunneo@smh.com.au",
                         "gruosslq@miitbeian.gov.cn",
                         "gdorantmv@rakuten.co.jp",
                         "tshemminnv@angelfire.com",
                         "kcoulsonge@state.tx.us",
                         "grouchynb@arizona.edu",
                         "mmilleaw@scientificamerican.com",
                         "wwiles26@sbwire.com",
                         "rstockings6x@wisc.edu",
                         "mfulgerat@jiathis.com",
                         "tgiovannacci8q@wix.com",
                         "aturbatks@gov.uk",
                         "crikkardl8@tamu.edu",
                         "tdegower1@sohu.com",
                         "mlockett2l@icio.us",
                         "iwilkinsonke@army.mil",
                         "olearmanr7@epa.gov",
                         "lpeetermannon@nps.gov",
                         "atilly7y@shutterfly.com" };

  const char* values[] = {
    "Afrikaans",     "Japanese",     "Albanian",   "French",
    "Malay",         "Czech",        "Dhivehi",    "Hiri Motu",
    "Bulgarian",     "Bosnian",      "Kyrgyz",     "Gagauz",
    "Polish",        "Swati",        "Hebrew",     "Georgian",
    "Pashto",        "Hindi",        "Dutch",      "Montenegrin",
    "Dzongkha",      "Indonesian",   "Italian",    "New Zealand Sign Language",
    "Latvian",       "Ndebele",      "Persian",    "Burmese",
    "Swedish",       "Greek",        "Kannada",    "Aymara",
    "Telugu",        "Danish",       "Gujarati",   "Armenian",
    "Swahili",       "Finnish",      "Macedonian", "Portuguese",
    "Hungarian",     "Irish Gaelic", "Khmer",      "Quechua",
    "Luxembourgish", "Zulu",         "Papiamento", "German",
    "Kurdish",       "Malagasy",     "Romanian",   "Marathi",
    "Norwegian",     "Sotho",        "Estonian",   "Tswana",
    "Filipino",      "Thai",         "Spanish",    "West Frisian",
    "Nepali",        "Malayalam",    "Croatian",   "Lithuanian",
    "Bislama",       "Bengali",      "Kashmiri",   "Tsonga",
    "Tamil",         "Lao",          "Chinese",    "Haitian Creole",
    "Azeri",         "Punjabi",      "Tok Pisin",  "Northern Sotho",
    "Catalan",       "Assamese",     "Arabic",     "Korean",
    "Yiddish",       "Amharic",      "Dari",       "Fijian",
    "Belarusian",    "Maltese",      "English",    "Icelandic",
    "Oriya",         "Moldovan",     "Mongolian",  "Somali",
    "Tajik",         "Kazakh",       "Tetum"
  };

  unsigned int nrvalues = sizeof(values) / sizeof(values[0]);
  unsigned int nrkeys = sizeof(keys) / sizeof(keys[0]);

  map_T* map = NEW_MAP();

  for (unsigned int i = 0; i < nrkeys; i++) {
    const char* k = keys[i];
    const char* v = values[i % nrvalues];

    map_set(map, strdup(k), strdup(v));
  }

  for (unsigned int i = 0; i < nrkeys; i++) {
    const char* k = keys[i];
    const char* v = values[i % nrvalues];

    map_bucket_T* item = map_get(map, (char*)k);

    if (!ASSERT(item != 0, "assert value in map.")) {
      return;
    }
    if (!ASSERT(strcmp((char*)item->value, v) == 0, "assert item has expected value.")) {
      return;
    }
  }

  PRINT_INFO();
}

void test_map_vs_loop()
{
  const char* keys[] = { "bdast6e@pinterest.com",
                         "syoellj6@opera.com",
                         "lyegoshin2c@squarespace.com",
                         "hwalne8w@ycombinator.com",
                         "mpech2v@columbia.edu",
                         "fstonhardby@seattletimes.com",
                         "asimonsot@indiatimes.com",
                         "czorzoniaj@youtube.com",
                         "fravel7h@netlog.com",
                         "mwaythingr9@yandex.ru",
                         "gbeert82@exblog.jp",
                         "hkocherri@forbes.com",
                         "jdray7b@un.org",
                         "mashbrookia@washingtonpost.com",
                         "dfardonix@ted.com",
                         "denriquesdu@answers.com",
                         "hmacci5p@ted.com",
                         "ecoltart8d@chron.com",
                         "cwiltshawh0@bing.com",
                         "glacyie@guardian.co.uk",
                         "tjammetm6@youtu.be",
                         "jwindlessjz@icq.com",
                         "ggrasonnk@canalblog.com",
                         "sspearrn@sohu.com",
                         "klawsb6@wikimedia.org",
                         "pmcshirieqb@livejournal.com",
                         "spickton5a@constantcontact.com",
                         "lmugglp@canalblog.com",
                         "sseresq8@feedburner.com",
                         "komoylan27@berkeley.edu",
                         "gmeersiv@adobe.com",
                         "fbraneykt@ihg.com",
                         "dfabbag@buzzfeed.com",
                         "fdankkk@ifeng.com",
                         "ejakemannd@skyrock.com",
                         "ybonnor67@ucoz.com",
                         "ncranmerecs@sphinn.com",
                         "abluneed@unicef.org",
                         "nblameyee@abc.net.au",
                         "rskonegi@google.fr",
                         "qdrogandh@moonfruit.com",
                         "sruspinebl@cnbc.com",
                         "mpeasbyqp@feedburner.com",
                         "ipillingergj@meetup.com",
                         "amithun37@oakley.com",
                         "cdavley68@mac.com",
                         "gmarples7u@ameblo.jp",
                         "lmungin8c@npr.org",
                         "mboots7w@elegantthemes.com",
                         "ltirrellis@51.la",
                         "sdumphreyshn@cnn.com",
                         "abodicumma@mapy.cz",
                         "ccrichley1z@mac.com",
                         "lurionh7@zdnet.com",
                         "sfey73@google.it",
                         "enansoner@berkeley.edu",
                         "gocrevy8f@goo.ne.jp",
                         "kspreyiy@ifeng.com",
                         "ccarlozzi61@sina.com.cn",
                         "mheitonor@vinaora.com",
                         "cpopleebi@sciencedirect.com",
                         "hpocklingtona1@google.nl",
                         "mantyshev5m@cloudflare.com",
                         "tsegoe5j@booking.com",
                         "ekillgusjq@mtv.com",
                         "dblundanjw@ameblo.jp",
                         "ndalrymplein@answers.com",
                         "hbentinckbw@soup.io",
                         "phammerberg1a@businessinsider.com",
                         "emettrickecr@google.com.au",
                         "amilby4y@sbwire.com",
                         "cdomnineyay@vistaprint.com",
                         "lbruna7@instagram.com",
                         "spaolozzi51@sohu.com",
                         "cgrinyerp8@examiner.com",
                         "swoodroofc@yahoo.com",
                         "fnorthrop50@mozilla.org",
                         "cgodrichew@netvibes.com",
                         "abarzenoz@army.mil",
                         "nfashame3@artisteer.com",
                         "besplin2o@nps.gov",
                         "dmargremij@home.pl",
                         "gteanby9r@g.co",
                         "cmathewson2i@constantcontact.com",
                         "wboschmann2x@pagesperso-orange.fr",
                         "vdowson23@discuz.net",
                         "mdunmore8g@163.com",
                         "ggroddenb1@fda.gov",
                         "tlinnelle5@jimdo.com",
                         "mpennycuickk5@furl.net",
                         "lduinbletonm2@t.co",
                         "sbrightgc@mayoclinic.com",
                         "trosenblath7l@house.gov",
                         "rpelzky@sitemeter.com",
                         "kbenoixiw@hatena.ne.jp",
                         "jpeadpn@japanpost.jp",
                         "efenbyet@sciencedirect.com",
                         "abalharrielm@youtu.be",
                         "ecolesrr@cdc.gov",
                         "bcollinghamam@google.de",
                         "cmuttockf8@paginegialle.it",
                         "ascrivner1d@symantec.com",
                         "bsoutherdenbu@latimes.com",
                         "bderringtonir@fda.gov",
                         "cleethemdb@adobe.com",
                         "lschusterq1@mail.ru",
                         "fmillyardpc@i2i.jp",
                         "mbriffett4c@hhs.gov",
                         "fmclugishro@toplist.cz",
                         "sbridiegz@reverbnation.com",
                         "jklimp6q@samsung.com",
                         "sbaudoneif@vk.com",
                         "rdoucettedg@go.com",
                         "ysonierho@cdc.gov",
                         "cyirrellhg@digg.com",
                         "ldenerp1@technorati.com",
                         "cangerstein8o@oracle.com",
                         "jlawillefy@i2i.jp",
                         "amadnerc7@eepurl.com",
                         "rachromovkw@shinystat.com",
                         "wscardifeild76@chron.com",
                         "lmatschoss9q@freewebs.com",
                         "vmoringg1@canalblog.com",
                         "wdelfqd@mit.edu",
                         "bantoniadesqt@vistaprint.com",
                         "hhinckesmanq4@kickstarter.com",
                         "fpillmanoy@ezinearticles.com",
                         "kgoulbornbr@cnet.com",
                         "dmelliardh@mac.com",
                         "fricketes@who.int",
                         "mmiddle4q@ox.ac.uk",
                         "cellicer@cloudflare.com",
                         "sbraundte8@psu.edu",
                         "tboerdermanb4@dmoz.org",
                         "boglassanepe@yale.edu",
                         "afreemantleka@blogs.com",
                         "akeppie6y@163.com",
                         "ejannawayb3@twitpic.com",
                         "kcaulier63@rediff.com",
                         "jgoodbannede@nationalgeographic.com",
                         "kfifoot5l@latimes.com",
                         "nmunciefz@epa.gov",
                         "mknaggsjx@shop-pro.jp",
                         "wmaccoughen6i@symantec.com",
                         "wlanningk8@sakura.ne.jp",
                         "rbreitlingfi@yolasite.com",
                         "lvalentimpp@wikipedia.org",
                         "nhaggisre@cdc.gov",
                         "lhauchdt@hexun.com",
                         "hthoriusgk@bluehost.com",
                         "tweinenf@sbwire.com",
                         "jteligine@typepad.com",
                         "jmortelj5@pen.io",
                         "adundridgeo3@theatlantic.com",
                         "jbenedettini8a@nationalgeographic.com",
                         "bmatijevicl@google.com.hk",
                         "mdelahunty@behance.net",
                         "achantera8@msu.edu",
                         "gpoxtonh2@zdnet.com",
                         "tmelwallkn@shinystat.com",
                         "tspekee4@dailymotion.com",
                         "dramsbothamn5@tmall.com",
                         "gshoesmith2w@com.com",
                         "htrelevanop@feedburner.com",
                         "eselwayda@tumblr.com",
                         "hburkino5@walmart.com",
                         "lgraysmarkio@privacy.gov.au",
                         "dskinnbp@pbs.org",
                         "cparkesonbt@reference.com",
                         "aviles45@live.com",
                         "llassenm4@yellowbook.com",
                         "mcholtongw@msn.com",
                         "ksteane5t@wisc.edu",
                         "aaxonk4@engadget.com",
                         "cjewesdq@wufoo.com",
                         "dsimonichj9@amazon.com",
                         "amcgowranp0@guardian.co.uk",
                         "ksterry42@aboutads.info",
                         "fwarind8@friendfeed.com",
                         "ncaskie7o@indiegogo.com",
                         "adelgadonz@mac.com",
                         "cdeioh@washington.edu",
                         "fballaam1q@purevolume.com",
                         "lpethybridge8s@patch.com",
                         "lstaker9d@360.cn",
                         "apatilloiq@homestead.com",
                         "bstickens28@seesaa.net",
                         "ccestardj2@sourceforge.net",
                         "sdewintonkm@mit.edu",
                         "greyburn4x@xrea.com",
                         "cjarrel3v@youtube.com",
                         "lallrightax@vk.com",
                         "kmashamar@slashdot.org",
                         "epreefd@rambler.ru",
                         "tpratore9@desdev.cn",
                         "jdominec92@princeton.edu",
                         "wmcwilliamscb@goo.gl",
                         "khonnicottrk@ovh.net",
                         "jlogue4i@uol.com.br",
                         "rarmatidgemw@creativecommons.org",
                         "ttaskerah@house.gov",
                         "bogersp6@blogspot.com",
                         "aarcher88@yellowbook.com",
                         "caverillm@oaic.gov.au",
                         "evasyukhnovi1@google.ru",
                         "cpinockn7@barnesandnoble.com",
                         "rdampct@dailymotion.com",
                         "jdosdelljg@freewebs.com",
                         "rjurczykmx@is.gd",
                         "nphaupfe@netscape.com",
                         "eharkinsof@whitehouse.gov",
                         "kdavidzonbx@bloglines.com",
                         "mbasire8i@desdev.cn",
                         "ccowlini2@bandcamp.com",
                         "aherreranoo@skype.com",
                         "inend31@dmoz.org",
                         "mcreadyqv@ifeng.com",
                         "ttaggertyae@wikispaces.com",
                         "mcinellijp@cnn.com",
                         "kmquhargepw@npr.org",
                         "emalamore1j@cisco.com",
                         "afilanrj@creativecommons.org",
                         "ccompson75@newsvine.com",
                         "cchristofidesnl@stanford.edu",
                         "lriddingq7@columbia.edu",
                         "jcirlos3b@ezinearticles.com",
                         "rdorney64@nsw.gov.au",
                         "ccrackel69@miibeian.gov.cn",
                         "pbuss86@paginegialle.it",
                         "bdankovl1@flickr.com",
                         "bduigenane1@simplemachines.org",
                         "preedyhough4k@bluehost.com",
                         "cmolsonab@tinyurl.com",
                         "rdockwrayef@ed.gov",
                         "jbosnellfo@skyrock.com",
                         "ccartmillom@prlog.org",
                         "telliottpf@sfgate.com",
                         "rstartin6r@amazon.com",
                         "gnussiinm@stanford.edu",
                         "evasnevjl@cocolog-nifty.com",
                         "sconnow3s@is.gd",
                         "ndudingaf@blogs.com",
                         "psparshottme@liveinternet.ru",
                         "ilarcierkf@t.co",
                         "escholig3y@unblog.fr",
                         "bhoolaghank9@reuters.com",
                         "dhebblewhitecp@opensource.org",
                         "mallbon98@timesonline.co.uk",
                         "knencim5@abc.net.au",
                         "kgibbesonok@dion.ne.jp",
                         "jbernardotte3g@ameblo.jp",
                         "idorricott9z@youtu.be",
                         "tstonesdw@i2i.jp",
                         "bgraffhamji@etsy.com",
                         "tluckingsa6@bluehost.com",
                         "bcutteridge5u@time.com",
                         "kbromwichpi@independent.co.uk",
                         "tbrantlmp@clickbank.net",
                         "wgrecel7@squarespace.com",
                         "jblackalllk@scientificamerican.com",
                         "sineson33@jiathis.com",
                         "tterbruggend6@nasa.gov",
                         "cgarrowayf1@java.com",
                         "mgyerpu@huffingtonpost.com",
                         "kmckmurriegq@bigcartel.com",
                         "mroscamps7x@flavors.me",
                         "mbasekep5@samsung.com",
                         "gpowrieqx@mapquest.com",
                         "rlazenbycz@wiley.com",
                         "romeara62@i2i.jp",
                         "pflipekp4@google.ru",
                         "bcolbert43@wikispaces.com",
                         "pvautrey5z@ebay.co.uk",
                         "tkislingburyd2@acquirethisname.com",
                         "etunneymo@tmall.com",
                         "mgalliardfv@360.cn",
                         "msadgroveb7@macromedia.com",
                         "ehelixmu@spotify.com",
                         "mleathard3j@histats.com",
                         "npieterlf@google.de",
                         "rsmale3m@disqus.com",
                         "hcapeling4e@woothemes.com",
                         "chincksjf@so-net.ne.jp",
                         "rstilgoekq@huffingtonpost.com",
                         "hpodbury3e@privacy.gov.au",
                         "escapensca@adobe.com",
                         "kkenealyi4@vinaora.com",
                         "mderricoatms@earthlink.net",
                         "rtipladyr2@homestead.com",
                         "ldownscw@blogspot.com",
                         "msimms10@aboutads.info",
                         "dwysome4h@phpbb.com",
                         "bburnyeaths@woothemes.com",
                         "dbennet4z@usda.gov",
                         "gwetherickig@so-net.ne.jp",
                         "croonyc9@hp.com",
                         "bwashbrook7k@examiner.com",
                         "stuft6o@seattletimes.com",
                         "sreppaportl5@cpanel.net",
                         "lgoricken0@nature.com",
                         "aocollopy2g@smh.com.au",
                         "eshellumkr@bloglovin.com",
                         "jgiloglylv@storify.com",
                         "fkuzemka49@cmu.edu",
                         "ebartell1k@goo.gl",
                         "nfrancklingu@google.co.uk",
                         "fjenkinsonle@scientificamerican.com",
                         "rblakesl0@alibaba.com",
                         "mgarment6h@joomla.org",
                         "cnendickai@goodreads.com",
                         "gphettisdd@netscape.com",
                         "echurmsgg@nbcnews.com",
                         "rhartgillrq@unesco.org",
                         "mpeasgood6n@vk.com",
                         "zbool2t@creativecommons.org",
                         "jisaacs6u@mail.ru",
                         "cskermanen@sogou.com",
                         "bwendover24@unicef.org",
                         "edreverb2@etsy.com",
                         "tbrissendenih@soup.io",
                         "vgianolohu@phpbb.com",
                         "marpino4f@linkedin.com",
                         "elanneyjc@walmart.com",
                         "bmcbrearty8x@google.com.hk",
                         "amarconi16@nhs.uk",
                         "gminshulli7@va.gov",
                         "askeermermn@php.net",
                         "ljeness3d@tripod.com",
                         "dcornforthr4@skype.com",
                         "gtreadgear4@netlog.com",
                         "jbogart5w@jugem.jp",
                         "lcultcheth6w@sun.com",
                         "abernolletad@github.com",
                         "yconen2h@netvibes.com",
                         "mbowdrey3l@noaa.gov",
                         "ematzeitispo@storify.com",
                         "dsidnall0@narod.ru",
                         "mschaumanngy@parallels.com",
                         "lamerici9v@sitemeter.com",
                         "gminton1r@nytimes.com",
                         "caudrittmi@smh.com.au",
                         "tcorsorod@theatlantic.com",
                         "rjephcotef0@dot.gov",
                         "gfinders3a@google.ru",
                         "emiramso2@berkeley.edu",
                         "jbrammerc4@mit.edu",
                         "tdegregoriop7@photobucket.com",
                         "kbrave58@ftc.gov",
                         "snowickf5@blogger.com",
                         "shetterich5r@vk.com",
                         "vstonerbj@people.com.cn",
                         "bmcwhorter3k@apple.com",
                         "llabincf@businessweek.com",
                         "idimondh5@paypal.com",
                         "pschowenburgh9@msu.edu",
                         "cjannequinqo@joomla.org",
                         "msemrad30@php.net",
                         "avasiliu9b@parallels.com",
                         "lfroomepb@sitemeter.com",
                         "dorringne@zimbio.com",
                         "smoulsdalekj@wiley.com",
                         "rmorrisonns@wsj.com",
                         "mcock6j@bandcamp.com",
                         "gpothecarylj@youtu.be",
                         "bmooringhk@sfgate.com",
                         "edrydaleit@goo.gl",
                         "cschottm7@aboutads.info",
                         "bhinstockjj@wix.com",
                         "lsheirlaw7z@boston.com",
                         "bmeriothqn@blogs.com",
                         "blanglandj1@free.fr",
                         "mgronouslr@xrea.com",
                         "ddechelle3r@google.fr",
                         "mdiggar57@google.co.uk",
                         "gdradeyei@gizmodo.com",
                         "bpiscopiellola@homestead.com",
                         "zrobkenhq@bloglines.com",
                         "awestpfel5n@sina.com.cn",
                         "cgoddardbk@uol.com.br",
                         "ddagwell93@infoseek.co.jp",
                         "moliviahz@dagondesign.com",
                         "nocklando9@cnet.com",
                         "fmckiddinh6@who.int",
                         "tguillartdm@irs.gov",
                         "rhayseyfu@webs.com",
                         "vkempeqc@plala.or.jp",
                         "lboote3t@hc360.com",
                         "codydr@storify.com",
                         "givshineg@telegraph.co.uk",
                         "rsykes96@java.com",
                         "dgrzegorzewicz6z@blinklist.com",
                         "bfrostd9@rakuten.co.jp",
                         "cdunlap54@mayoclinic.com",
                         "vsang2s@blogs.com",
                         "ghumphrey5e@dailymail.co.uk",
                         "mgerrett18@google.nl",
                         "vkynnd2j@yellowpages.com",
                         "ghemshalleq@nba.com",
                         "nmarrinerjb@guardian.co.uk",
                         "hturevillem1@cafepress.com",
                         "irickhuss66@shutterfly.com",
                         "rloaldayic@go.com",
                         "gsydallju@businesswire.com",
                         "lkellscq@alibaba.com",
                         "iedlerfr@accuweather.com",
                         "csiegequ@nbcnews.com",
                         "dbathersby4g@lulu.com",
                         "kbeldon8r@nytimes.com",
                         "hwisonkx@google.co.uk",
                         "adanielibf@ifeng.com",
                         "grhydderchnr@uiuc.edu",
                         "pmccreadyr6@xing.com",
                         "smacounk2@constantcontact.com",
                         "nroobottom95@dagondesign.com",
                         "mpoonebc@cnet.com",
                         "mgorbelll9@fda.gov",
                         "pramln@jugem.jp",
                         "opackingtonj0@unicef.org",
                         "hmcbethj8@desdev.cn",
                         "aprandygv@reverbnation.com",
                         "trosenkrantza2@amazonaws.com",
                         "lhakerhw@usnews.com",
                         "skettlesingqh@booking.com",
                         "vamber41@apache.org",
                         "ajasperfp@bravesites.com",
                         "wbeceril44@princeton.edu",
                         "tlewsley4n@nature.com",
                         "blavissrl@mail.ru",
                         "crosenfieldil@exblog.jp",
                         "emar8n@bloomberg.com",
                         "hllopisdi@cornell.edu",
                         "gforgan85@addthis.com",
                         "nfuzzeyl2@businessinsider.com",
                         "kkincaidmd@scientificamerican.com",
                         "ccalytond@usgs.gov",
                         "jrailton91@blogs.com",
                         "dlauranthv@nymag.com",
                         "bsherrardja@alexa.com",
                         "kpichanbo@home.pl",
                         "jmordykg@independent.co.uk",
                         "cballe4r@cnbc.com",
                         "ldigmanls@seattletimes.com",
                         "eslamakerba@google.com.au",
                         "kgodspeede2e@ustream.tv",
                         "izanettiii@yahoo.co.jp",
                         "rcoultardi3@aboutads.info",
                         "mbilovus5f@i2i.jp",
                         "fpleasauncebq@cbslocal.com",
                         "asargeauntpl@dion.ne.jp",
                         "jgligorijevic9o@walmart.com",
                         "dgottschalkqg@wordpress.com",
                         "ivanderwaltff@yellowbook.com",
                         "gdoey55@who.int",
                         "elittlecote5v@pcworld.com",
                         "kespinpg@deviantart.com",
                         "dsailrm@globo.com",
                         "zbaston3h@tmall.com",
                         "adils8t@cam.ac.uk",
                         "dworsfield9n@cloudflare.com",
                         "nwardesworth17@about.com",
                         "stinkhambn@altervista.org",
                         "jandreidj@is.gd",
                         "tsutterbyo0@hc360.com",
                         "hmehargp2@admin.ch",
                         "eburghalljd@slideshare.net",
                         "ymckearnen71@domainmarket.com",
                         "jdewick77@shinystat.com",
                         "ukirkbridelo@github.com",
                         "dmacgarveyje@cbsnews.com",
                         "dbettesworthk1@altervista.org",
                         "sdilletn4@vk.com",
                         "sstarte9x@netscape.com",
                         "rfoxleypy@ox.ac.uk",
                         "omilillocy@craigslist.org",
                         "hduggena0@mashable.com",
                         "bbednallmz@miitbeian.gov.cn",
                         "nbattams4p@microsoft.com",
                         "mbutterley2a@prweb.com",
                         "rprangleak@bloglovin.com",
                         "aworttny@prlog.org",
                         "sbalsellie3z@tmall.com",
                         "ckillingbackft@dion.ne.jp",
                         "bgrievson89@freewebs.com",
                         "msmeesbv@statcounter.com",
                         "asodoi5@exblog.jp",
                         "mtaitec@wordpress.org",
                         "vrasselllw@newsvine.com",
                         "vnicol74@ifeng.com",
                         "fstrongen2@infoseek.co.jp",
                         "jbootellg9@salon.com",
                         "aplunketgb@jigsy.com",
                         "tcicci4l@state.gov",
                         "dpinchinglc@craigslist.org",
                         "ahusk1u@google.de",
                         "bmundow9i@deviantart.com",
                         "hroustki@umich.edu",
                         "kbrandi79@joomla.org",
                         "wlilleycrop65@newsvine.com",
                         "chuotaw@squarespace.com",
                         "eguernerc6@baidu.com",
                         "dkerblerck@usnews.com",
                         "ehanalanav@facebook.com",
                         "cwykemq@nydailynews.com",
                         "rspeere3@scientificamerican.com",
                         "hdecristoforofx@vimeo.com",
                         "emckirtonhh@dailymotion.com",
                         "frallinpa@spiegel.de",
                         "glakesap@over-blog.com",
                         "zolleta3@abc.net.au",
                         "btrusse3n@nationalgeographic.com",
                         "tlowesgs@wikia.com",
                         "vcowelyc0@wix.com",
                         "demlyn7t@opensource.org",
                         "pfredianimy@cnbc.com",
                         "aandoverpm@dailymail.co.uk",
                         "tfarres2k@bbb.org",
                         "vkenyamdx@tripadvisor.com",
                         "ecollidge1x@jalbum.net",
                         "bwiggall3w@163.com",
                         "mduckerinoe@ow.ly",
                         "mlamartinegr@nba.com",
                         "hvallery9s@amazon.de",
                         "draiman5@theguardian.com",
                         "htassellds@wikia.com",
                         "mashpolep@loc.gov",
                         "wpeasey29@ycombinator.com",
                         "jdaulbyha@aol.com",
                         "ldeluca5o@rakuten.co.jp",
                         "egreenwood4b@kickstarter.com",
                         "fpluckj3@nifty.com",
                         "abraderl4@reverbnation.com",
                         "nschenkelmr@comsenz.com",
                         "szecchinio8@booking.com",
                         "kbiaggelli1h@w3.org",
                         "ijemisonx@washington.edu",
                         "dcopelli2u@businessweek.com",
                         "wjubblx@sitemeter.com",
                         "lcasaroli9k@csmonitor.com",
                         "rlawranson7f@reference.com",
                         "kalkerb@sina.com.cn",
                         "cmetzing7n@uiuc.edu",
                         "mgarred25@timesonline.co.uk",
                         "nreglara4@yellowbook.com",
                         "earnaldy53@bloomberg.com",
                         "kgwinn36@cornell.edu",
                         "bpennamanf@microsoft.com",
                         "csquirrell78@newyorker.com",
                         "cwyrek7@businesswire.com",
                         "rindruchn@oracle.com",
                         "bcarrellq0@hostgator.com",
                         "adaftorrg@blogspot.com",
                         "amccrossandl@army.mil",
                         "emartijnj@nasa.gov",
                         "cbadgersld@tmall.com",
                         "cduesbury5d@ucoz.ru",
                         "ewoolhouseep@people.com.cn",
                         "cjorczykjt@geocities.jp",
                         "bolfertmf@comsenz.com",
                         "kwellandc5@jugem.jp",
                         "gclaymj@dailymotion.com",
                         "gnormanvilleey@comcast.net",
                         "hthexton2n@wsj.com",
                         "hhoodlassjy@oaic.gov.au",
                         "dpicton2m@cmu.edu",
                         "kmilbankg4@parallels.com",
                         "acarrollhb@umn.edu",
                         "ddivine2y@archive.org",
                         "alunnqq@qq.com",
                         "dfruchond7@arstechnica.com",
                         "dcoviellog5@so-net.ne.jp",
                         "dchristophersgh@economist.com",
                         "ldyneleyqz@lycos.com",
                         "dgrowden7v@google.cn",
                         "ecolebecknt@ycombinator.com",
                         "rmccluskeyqs@wordpress.org",
                         "jlillegard4u@google.com",
                         "jcattermull5s@ustream.tv",
                         "vjindracekk6@illinois.edu",
                         "bregnardpq@squarespace.com",
                         "jschaffeldy@springer.com",
                         "bspohrmannjo@newsvine.com",
                         "akepling2b@bandcamp.com",
                         "mscollardg2@nymag.com",
                         "ybrideau97@earthlink.net",
                         "kpointing6s@statcounter.com",
                         "cgowland8b@youtu.be",
                         "sbendik15@umn.edu",
                         "mdimentgd@paypal.com",
                         "kwolfendell3u@economist.com",
                         "brisbie9f@multiply.com",
                         "jwealthallaa@alexa.com",
                         "csoutenhp@nationalgeographic.com",
                         "ihackforthi6@deliciousdays.com",
                         "jballinl6@livejournal.com",
                         "tdaltreya5@goo.ne.jp",
                         "mtempleton1c@businesswire.com",
                         "thallward7d@aol.com",
                         "scockertone6@google.pl",
                         "sbecraftmb@wikia.com",
                         "dtrumblee7@virginia.edu",
                         "dkettlestingkh@plala.or.jp",
                         "dchalkbz@yolasite.com",
                         "mmcquorkel6k@nih.gov",
                         "dmurfett2@livejournal.com",
                         "lerskin1b@cornell.edu",
                         "parntzenjk@e-recht24.de",
                         "rcaberapt@wunderground.com",
                         "cjakubovicz3i@mail.ru",
                         "kstefanovicel@samsung.com",
                         "bdyson7g@istockphoto.com",
                         "scowxa@upenn.edu",
                         "dgowman94@wikipedia.org",
                         "ngladtbachdo@sakura.ne.jp",
                         "smilsomgl@discuz.net",
                         "achild5i@forbes.com",
                         "sackensonnx@studiopress.com",
                         "bronchka12@discuz.net",
                         "dwison2r@printfriendly.com",
                         "slockneral@umn.edu",
                         "ilenahan6t@webnode.com",
                         "kayrtondv@forbes.com",
                         "sivattsoc@yandex.ru",
                         "slatehouse4w@ocn.ne.jp",
                         "smargerrison1m@dagondesign.com",
                         "mmingusrh@cbc.ca",
                         "dbeddofa@wiley.com",
                         "rshard2q@harvard.edu",
                         "jmanselleh@java.com",
                         "xcaillouhm@123-reg.co.uk",
                         "mbrise1f@latimes.com",
                         "gsayeshf@hugedomains.com",
                         "cdeckerm8@mapquest.com",
                         "camblerg8@nba.com",
                         "jkarim1i@nytimes.com",
                         "heveleighk0@webeden.co.uk",
                         "lneweynq@lycos.com",
                         "emounceycu@gravatar.com",
                         "pbirdfieldgn@house.gov",
                         "spourveer8z@blog.com",
                         "shuggettn8@mysql.com",
                         "mchithamqy@surveymonkey.com",
                         "ajacop2d@slideshare.net",
                         "vreubbensaq@usnews.com",
                         "pknowltoni8@geocities.jp",
                         "dbeaj7@hugedomains.com",
                         "lmolangm@so-net.ne.jp",
                         "wpykettk3@fda.gov",
                         "bbaynsi@csmonitor.com",
                         "tsowood56@wix.com",
                         "ltutin5b@java.com",
                         "lorneblowm9@wiley.com",
                         "gcrawshayh1@opensource.org",
                         "vchasmor7p@purevolume.com",
                         "rlarkby8v@1und1.de",
                         "lgreenset13@hao123.com",
                         "hvatcherbb@facebook.com",
                         "mszwandtej@china.com.cn",
                         "ktuiteik@alibaba.com",
                         "dgyngylln9@hp.com",
                         "ssprowsonnu@fc2.com",
                         "mgarnsworthyqj@dropbox.com",
                         "stwigg22@archive.org",
                         "rcaramuscia8y@princeton.edu",
                         "pbrandimm@indiegogo.com",
                         "ddawks7a@oakley.com",
                         "lrounds2p@virginia.edu",
                         "ewatson32@soup.io",
                         "bmicahjn@usnews.com",
                         "kbucktrout9u@delicious.com",
                         "mfuryov@imdb.com",
                         "lmcconnellc2@slashdot.org",
                         "rwhetsonrd@hp.com",
                         "rbrindedf7@imdb.com",
                         "skevishi@examiner.com",
                         "mwardhawkb@bloglines.com",
                         "llantiff3o@house.gov",
                         "aspurrettng@wix.com",
                         "jmacneishmt@wikipedia.org",
                         "odjurisicn3@google.pl",
                         "gyouell6a@cdbaby.com",
                         "jthirkettlefl@usatoday.com",
                         "seastbymc@springer.com",
                         "pdemetrqf@psu.edu",
                         "cdelgardob0@apple.com",
                         "hmaclisef2@bizjournals.com",
                         "ltwycross8m@apple.com",
                         "mbulfoyd0@photobucket.com",
                         "shyde20@yelp.com",
                         "sfaraganv@uiuc.edu",
                         "aforstallnw@aol.com",
                         "kgentzsch39@google.nl",
                         "tgarfitt83@stanford.edu",
                         "galderseyao@sakura.ne.jp",
                         "sstollencm@tumblr.com",
                         "iivermeedn@cbc.ca",
                         "wmacalpyneq2@abc.net.au",
                         "askillett6f@devhub.com",
                         "krusean@ehow.com",
                         "tmealiffeno@newsvine.com",
                         "tbriteec8@joomla.org",
                         "aganeph@blogger.com",
                         "mtodeogp@usgs.gov",
                         "jvallens84@theglobeandmail.com",
                         "asharman59@imgur.com",
                         "pmccollumow@ask.com",
                         "rcoade2f@naver.com",
                         "cluckings70@walmart.com",
                         "ahonackerdf@paginegialle.it",
                         "kburyfm@joomla.org",
                         "joliddyli@usnews.com",
                         "edeleekkd@ameblo.jp",
                         "mcollarj4@alexa.com",
                         "htuxselllu@ca.gov",
                         "jstickingslh@technorati.com",
                         "mghilardi81@storify.com",
                         "pleathard1w@wired.com",
                         "lfortinos@bizjournals.com",
                         "nhonniebaleu@auda.org.au",
                         "nlethbridgeq@dailymail.co.uk",
                         "jtowson4d@ustream.tv",
                         "cwrathmallo@cdc.gov",
                         "mbushnellc3@google.co.uk",
                         "wgullickhy@simplemachines.org",
                         "loxenford6b@whitehouse.gov",
                         "estourtonly@home.pl",
                         "tprottq3@360.cn",
                         "kmattionibg@flickr.com",
                         "kkingswoodcv@rediff.com",
                         "njanowski4j@umich.edu",
                         "shourstonmh@blogspot.com",
                         "mobeypj@amazon.de",
                         "gwigan9j@yellowbook.com",
                         "cbudnikrb@soundcloud.com",
                         "lbambercj@dion.ne.jp",
                         "ecaukillh3@usnews.com",
                         "amaingotib@hostgator.com",
                         "kjatczakcc@bloglovin.com",
                         "agrenedc@princeton.edu",
                         "vheish4@usnews.com",
                         "galdredg7@mapy.cz",
                         "wandrickga@ebay.co.uk",
                         "keardleyjr@spotify.com",
                         "jzealander5q@soundcloud.com",
                         "vkrolak40@intel.com",
                         "elermouth1g@wiley.com",
                         "nfreschidk@livejournal.com",
                         "calster7c@woothemes.com",
                         "mmowenpx@pinterest.com",
                         "gchongim@sciencedirect.com",
                         "aileyo1@biblegateway.com",
                         "adashkovra@e-recht24.de",
                         "ksteptoac@woothemes.com",
                         "lgilbertbh@utexas.edu",
                         "grihanekiu@mayoclinic.com",
                         "nkiltie9p@un.org",
                         "mveillardmk@wp.com",
                         "bmastertonfh@baidu.com",
                         "eceneyox@weebly.com",
                         "bwhissellk@umn.edu",
                         "dcollingridget@squidoo.com",
                         "asoal14@nifty.com",
                         "uwetheredd4@google.fr",
                         "wdunge8k@sitemeter.com",
                         "sgarmonjv@upenn.edu",
                         "rskenemg@disqus.com",
                         "bsammons9g@google.com.hk",
                         "liiannonikl@alexa.com",
                         "hscalerap3@themeforest.net",
                         "vrarityqe@oracle.com",
                         "cflawncg@php.net",
                         "esambrooklt@unesco.org",
                         "atindallco@discovery.com",
                         "hhackey4s@quantcast.com",
                         "gcobbe46@xing.com",
                         "gbunson3f@gov.uk",
                         "sgenteryf4@1688.com",
                         "ephizakarleyoj@nydailynews.com",
                         "zmaingot3q@meetup.com",
                         "kpattersa9@jigsy.com",
                         "dphysickf9@unc.edu",
                         "ogrossman21@google.com",
                         "ntawse9m@comsenz.com",
                         "evanleeuwen35@desdev.cn",
                         "glittqk@stumbleupon.com",
                         "lgenge6@skyrock.com",
                         "gkubatschog@biglobe.ne.jp",
                         "cfisbburne4m@discuz.net",
                         "dfirmage6v@mapquest.com",
                         "jabisettib5@cbslocal.com",
                         "lfawdriegf@mozilla.org",
                         "gcannonhr@ow.ly",
                         "fholseyku@google.cn",
                         "olaflinni@liveinternet.ru",
                         "csallengerqi@marketwatch.com",
                         "mwildenm0@wired.com",
                         "vbellayps@state.gov",
                         "mdudlestone5g@fc2.com",
                         "cmauntonq5@rediff.com",
                         "kscollandqm@ustream.tv",
                         "abrodbin1t@purevolume.com",
                         "csirette6m@skyrock.com",
                         "ddelmontikp@europa.eu",
                         "fstothartnc@reddit.com",
                         "vweightev@wordpress.com",
                         "fsigwardcl@behance.net",
                         "clufkin34@irs.gov",
                         "ahutley1e@opensource.org",
                         "cloynsch@fotki.com",
                         "kgallenkv@1und1.de",
                         "lsharplyl3@yellowbook.com",
                         "edrezzer52@soup.io",
                         "emonktong@phoca.cz",
                         "mokiei0@google.it",
                         "mblaiseez@ucla.edu",
                         "ilogesdalehe@wsj.com",
                         "kharkinsh8@unblog.fr",
                         "rhedworthfs@prweb.com",
                         "ijadonn@foxnews.com",
                         "dsanchizfq@irs.gov",
                         "jsouthernfk@elpais.com",
                         "mstrathdeehd@goo.gl",
                         "cbastianu@github.io",
                         "bjarley80@a8.net",
                         "brosensaftf6@washington.edu",
                         "gdeverick47@ameblo.jp",
                         "pfoalece@indiegogo.com",
                         "rprewer4t@odnoklassniki.ru",
                         "dingremn1@hhs.gov",
                         "rswatlandhc@youku.com",
                         "cgrindellci@cargocollective.com",
                         "froussellr5@pagesperso-orange.fr",
                         "bstandevenbm@chicagotribune.com",
                         "lwallbridgedp@unblog.fr",
                         "jpepye7i@gov.uk",
                         "wbatie9c@who.int",
                         "tlettucee0@github.io",
                         "vpadilla7q@bigcartel.com",
                         "dcreaghrf@netvibes.com",
                         "ccrackel72@amazonaws.com",
                         "mginiei9@intel.com",
                         "rdobbsem@wp.com",
                         "mhosbyfw@ustream.tv",
                         "wbowler8j@weather.com",
                         "ddunbletonpk@nih.gov",
                         "nstebbinml@deliciousdays.com",
                         "kbehrendsenq6@cocolog-nifty.com",
                         "jwixey7s@blogspot.com",
                         "ltolchargt@hubpages.com",
                         "sdoeylz@earthlink.net",
                         "danthonyd3@newyorker.com",
                         "mbootynj@woothemes.com",
                         "sverheydenr3@arstechnica.com",
                         "morwing3@columbia.edu",
                         "bdabels9y@wunderground.com",
                         "wupstonql@wordpress.com",
                         "egladwellll@goo.ne.jp",
                         "ccosker8p@usnews.com",
                         "vrenzullo1n@theglobeandmail.com",
                         "alowtherg0@mapquest.com",
                         "kstandleykz@nsw.gov.au",
                         "bkubczak6l@amazonaws.com",
                         "lredbourn3x@cbc.ca",
                         "bsergentcn@google.com.hk",
                         "jbeatiz@state.tx.us",
                         "rstampfer9t@creativecommons.org",
                         "dwhaley7e@zimbio.com",
                         "emegsonex@wunderground.com",
                         "stowne38@newyorker.com",
                         "fellisso7@geocities.com",
                         "mdelcastel8h@cdc.gov",
                         "rwegener8e@illinois.edu",
                         "kfrisbypv@woothemes.com",
                         "hcaltun7@independent.co.uk",
                         "edimblebyht@pcworld.com",
                         "lbenzab9@intel.com",
                         "eporkerna@seattletimes.com",
                         "bthaineip@gnu.org",
                         "jabrahamowitcz1@miitbeian.gov.cn",
                         "hbridgettkc@tiny.cc",
                         "dganterq9@home.pl",
                         "ejeffcoate1o@virginia.edu",
                         "zegginsek@pagesperso-orange.fr",
                         "iastillfg@deviantart.com",
                         "fjoutapaitisn6@csmonitor.com",
                         "gwoolfootjm@sbwire.com",
                         "dcocklin5c@storify.com",
                         "gseabrookee2@dyndns.org",
                         "epoyntergo@unc.edu",
                         "eocorhane7m@over-blog.com",
                         "fcrawshay9@go.com",
                         "cmaydwaycx@cbsnews.com",
                         "mfrandsen1s@cbsnews.com",
                         "hflewin8l@nsw.gov.au",
                         "mquan7j@soundcloud.com",
                         "kcorballyfc@hugedomains.com",
                         "gpookenp@dell.com",
                         "mheitonol@oaic.gov.au",
                         "lbartlemanoq@msn.com",
                         "lbenhamrc@webmd.com",
                         "arofes@samsung.com",
                         "kbaisepd@biblegateway.com",
                         "msaladino6g@imdb.com",
                         "scommanderlb@mayoclinic.com",
                         "pmityashevpz@opera.com",
                         "gdykasg6@cyberchimps.com",
                         "mfominov9h@wisc.edu",
                         "mofinanea@xrea.com",
                         "rpawling8@privacy.gov.au",
                         "lzannini5y@nps.gov",
                         "ckelliebd@blogspot.com",
                         "lheddeo6@intel.com",
                         "ahessp9@flickr.com",
                         "mnisco1l@wikia.com",
                         "lkeegan4a@un.org",
                         "cgirt1y@icio.us",
                         "msilmanhx@printfriendly.com",
                         "csterzakerr0@utexas.edu",
                         "jesby4v@desdev.cn",
                         "lrubbert9l@addthis.com",
                         "llowndesob@edublogs.org",
                         "ttanswell1p@google.com.au",
                         "dvanin19@fema.gov",
                         "alinec1@squarespace.com",
                         "lgarriockeb@g.co",
                         "kmacgeffen6d@dropbox.com",
                         "ihyner8@sourceforge.net",
                         "goverelko@desdev.cn",
                         "aeggle2z@illinois.edu",
                         "rfolomkin7r@gmpg.org",
                         "tnavein9e@acquirethisname.com",
                         "bfernelyd5@cnn.com",
                         "zdunlop8u@imageshack.us",
                         "bbelshawlg@cam.ac.uk",
                         "abrickellm3@shinystat.com",
                         "pstobbs9w@feedburner.com",
                         "efarensas@so-net.ne.jp",
                         "iravelusrp@omniture.com",
                         "htuddallhj@utexas.edu",
                         "nmacenzy6p@amazonaws.com",
                         "achristophersonaz@cam.ac.uk",
                         "dgomar5x@163.com",
                         "pfardo99@adobe.com",
                         "mclouston11@shutterfly.com",
                         "adelacote6c@biblegateway.com",
                         "dpalleske90@wordpress.com",
                         "mjuanes9a@tripod.com",
                         "mgwillymfj@thetimes.co.uk",
                         "nberickid@photobucket.com",
                         "cnassau1v@newyorker.com",
                         "cizardz@multiply.com",
                         "edahlgrenqr@ibm.com",
                         "tlatedz@princeton.edu",
                         "htabbitt60@mashable.com",
                         "ybyarscd@tripod.com",
                         "hvallintinefn@soundcloud.com",
                         "pwapplingtonqw@narod.ru",
                         "ikeasey4o@sun.com",
                         "lpopleyou@arstechnica.com",
                         "llinnejs@shop-pro.jp",
                         "gheiton87@fda.gov",
                         "sordelt3p@wufoo.com",
                         "idelacroix5k@nps.gov",
                         "mkitchingd1@marriott.com",
                         "bparisshl@addthis.com",
                         "bmccannyoi@nature.com",
                         "hvolagebs@printfriendly.com",
                         "thastqa@ustream.tv",
                         "dboldpr@psu.edu",
                         "rbelham5h@archive.org",
                         "rmencob8@illinois.edu",
                         "jmulberryf3@arizona.edu",
                         "dpflegero4@oakley.com",
                         "rbaileyfb@cloudflare.com",
                         "governh@qq.com",
                         "rsongist48@wufoo.com",
                         "cmargaretau@nsw.gov.au",
                         "sivain3c@aol.com",
                         "hlortzoa@economist.com",
                         "iprichardgx@tamu.edu",
                         "gluciusbe@acquirethisname.com",
                         "fboswardjh@myspace.com",
                         "ghunneo@smh.com.au",
                         "gruosslq@miitbeian.gov.cn",
                         "gdorantmv@rakuten.co.jp",
                         "tshemminnv@angelfire.com",
                         "kcoulsonge@state.tx.us",
                         "grouchynb@arizona.edu",
                         "mmilleaw@scientificamerican.com",
                         "wwiles26@sbwire.com",
                         "rstockings6x@wisc.edu",
                         "mfulgerat@jiathis.com",
                         "tgiovannacci8q@wix.com",
                         "aturbatks@gov.uk",
                         "crikkardl8@tamu.edu",
                         "tdegower1@sohu.com",
                         "mlockett2l@icio.us",
                         "iwilkinsonke@army.mil",
                         "olearmanr7@epa.gov",
                         "lpeetermannon@nps.gov",
                         "atilly7y@shutterfly.com" };

  const char* values[] = {
    "Afrikaans",     "Japanese",     "Albanian",   "French",
    "Malay",         "Czech",        "Dhivehi",    "Hiri Motu",
    "Bulgarian",     "Bosnian",      "Kyrgyz",     "Gagauz",
    "Polish",        "Swati",        "Hebrew",     "Georgian",
    "Pashto",        "Hindi",        "Dutch",      "Montenegrin",
    "Dzongkha",      "Indonesian",   "Italian",    "New Zealand Sign Language",
    "Latvian",       "Ndebele",      "Persian",    "Burmese",
    "Swedish",       "Greek",        "Kannada",    "Aymara",
    "Telugu",        "Danish",       "Gujarati",   "Armenian",
    "Swahili",       "Finnish",      "Macedonian", "Portuguese",
    "Hungarian",     "Irish Gaelic", "Khmer",      "Quechua",
    "Luxembourgish", "Zulu",         "Papiamento", "German",
    "Kurdish",       "Malagasy",     "Romanian",   "Marathi",
    "Norwegian",     "Sotho",        "Estonian",   "Tswana",
    "Filipino",      "Thai",         "Spanish",    "West Frisian",
    "Nepali",        "Malayalam",    "Croatian",   "Lithuanian",
    "Bislama",       "Bengali",      "Kashmiri",   "Tsonga",
    "Tamil",         "Lao",          "Chinese",    "Haitian Creole",
    "Azeri",         "Punjabi",      "Tok Pisin",  "Northern Sotho",
    "Catalan",       "Assamese",     "Arabic",     "Korean",
    "Yiddish",       "Amharic",      "Dari",       "Fijian",
    "Belarusian",    "Maltese",      "English",    "Icelandic",
    "Oriya",         "Moldovan",     "Mongolian",  "Somali",
    "Tajik",         "Kazakh",       "Tetum"
  };

  unsigned int nrvalues = sizeof(values) / sizeof(values[0]);
  unsigned int nrkeys = sizeof(keys) / sizeof(keys[0]);

  map_T* map = NEW_MAP();
  list_T* list = init_list(sizeof(list_kv_T*));

  for (unsigned int i = 0; i < nrkeys; i++) {
    const char* k = keys[i];
    const char* v = values[i % nrvalues];

    list_push(list, init_list_kv((char*)k, strdup(v)));

    map_set(map, strdup(k), strdup(v));
  }

  unsigned long int old = now();
  map_bucket_T* bucket = map_get(map, "aturbatks@gov.uk");
  unsigned long int new_time = now();
  ASSERT(bucket != 0, "bucket exists");
  printf("Lookup in map took: %ld ms.\n", new_time - old);

  list_kv_T* kv = 0;

  unsigned long int old2 = now();
  for (unsigned int i = 0; i < list->size; i++) {
    kv = (list_kv_T*)list->items[i];
    if (strcmp(kv->key, "aturbatks@gov.uk") == 0)
      break;
  }
  unsigned long int new_time2 = now();
  ASSERT(kv != 0, "key value exists");
  printf("Lookup in list took: %ld ms.\n", new_time2 - old2);
}

void test_map_unset()
{
  map_T* map = NEW_MAP();
  map_set(map, "xyz", strdup("123"));

  map_bucket_T* bucket = map_get(map, "xyz");
  ASSERT(bucket != 0, "item exists.");

  map_unset(map, "xyz");

  bucket = map_get(map, "xyz");
  ASSERT(bucket == 0, "item is removed.");
}

void test_map_unset_then_set()
{
  map_T* map = NEW_MAP();

  // set
  map_set(map, "xyz", strdup("123"));
  map_bucket_T* bucket = map_get(map, "xyz");
  ASSERT(bucket != 0, "item exists.");

  // unset
  map_unset(map, "xyz");

  // assert not exist
  bucket = map_get(map, "xyz");
  ASSERT(bucket == 0, "item is removed.");

  // assert exists again
  map_set(map, "xyz", strdup("123"));
  bucket = map_get(map, "xyz");
  ASSERT(bucket != 0, "item exists again.");
}

void test_map_get_keys()
{
  const char* keys[] = { "paradigm",
                         "Optional",
                         "access",
                         "focus group",
                         "Polarised",
                         "client-server",
                         "Fully-configurable",
                         "Fundamental",
                         "core",
                         "local area network",
                         "Customer-focused",
                         "Streamlined",
                         "service-desk",
                         "policy",
                         "structure",
                         "demand-driven",
                         "matrix",
                         "Progressive",
                         "zero defect",
                         "object-oriented",
                         "Open-architected",
                         "Re-contextualized",
                         "data-warehouse",
                         "needs-based",
                         "superstructure",
                         "Function-based",
                         "Implemented",
                         "systemic",
                         "groupware",
                         "Reactive",
                         "collaboration",
                         "productivity",
                         "De-engineered",
                         "asymmetric",
                         "portal",
                         "hybrid",
                         "regional",
                         "3rd generation",
                         "Centralized",
                         "mission-critical",
                         "discrete",
                         "orchestration",
                         "full-range",
                         "grid-enabled",
                         "optimal",
                         "alliance",
                         "Future-proofed",
                         "User-friendly",
                         "client-driven",
                         "Integrated",
                         "strategy",
                         "hierarchy",
                         "impactful",
                         "Multi-lateral",
                         "scalable",
                         "info-mediaries",
                         "homogeneous",
                         "foreground",
                         "projection",
                         "moderator",
                         "ability",
                         "Universal",
                         "transitional",
                         "installation",
                         "bi-directional",
                         "contextually-based",
                         "Virtual",
                         "5th generation",
                         "Exclusive",
                         "algorithm",
                         "upward-trending",
                         "knowledge base",
                         "Cross-platform",
                         "Optimized",
                         "product",
                         "incremental",
                         "composite",
                         "Monitored",
                         "infrastructure",
                         "Profit-focused",
                         "executive",
                         "website",
                         "neural-net",
                         "concept",
                         "intangible" };

  const char* values[] = {
    "Ophioglossaceae",  "Cyperaceae",    "Chenopodiaceae",   "Poaceae",          "Convolvulaceae",
    "Nymphaeaceae",     "Moraceae",      "Scrophulariaceae", "Opegraphaceae",    "Fabaceae",
    "Rhamnaceae",       "Fabaceae",      "Rubiaceae",        "Fabaceae",         "Fabaceae",
    "Teloschistaceae",  "Cucurbitaceae", "Apocynaceae",      "Scrophulariaceae", "Pteridaceae",
    "Dicranaceae",      "Poaceae",       "Campanulaceae",    "Teloschistaceae",  "Ranunculaceae",
    "Rubiaceae",        "Fabaceae",      "Cyperaceae",       "Fabaceae",         "Roccellaceae",
    "Aristolochiaceae", "Acanthaceae",   "Cactaceae",        "Myrtaceae",        "Adoxaceae",
    "Cyperaceae",       "Apocynaceae",   "Chenopodiaceae",   "Euphorbiaceae",    "Fabaceae",
    "Salicaceae",       "Asteraceae",    "Apiaceae",         "Lecanoraceae",     "Rosaceae",
    "Vitaceae",         "Thymelaeaceae", "Primulaceae",      "Liliaceae",        "Scrophulariaceae",
    "Polygonaceae",     "Violaceae",     "Verbenaceae",      "Poaceae",          "Lamiaceae"
  };

  unsigned int nrkeys = sizeof(keys) / sizeof(keys[0]);
  unsigned int nrvalues = sizeof(values) / sizeof(values[0]);

  map_T* map = NEW_MAP();

  for (unsigned int i = 0; i < nrkeys; i++) {
    const char* k = keys[i];
    const char* v = values[i % nrvalues];

    map_set(map, (char*)k, strdup(v));
  }

  char** keyz = 0;
  unsigned int _nrkeys;

  map_get_keys(map, &keyz, &_nrkeys);

  printf("%d actual: %d\n", _nrkeys, nrkeys);
  ASSERT(_nrkeys == nrkeys, "assert number of keys are correct");

  PRINT_INFO();
}

int main(int argc, char* argv[])
{
  test_map_set();
  test_map_get();
  test_map_multiple();
  test_map_change_value();
  test_map_resize();
  test_map_unset();
  test_map_unset_then_set();
  test_map_vs_loop();
  test_with_structs();
  test_map_get_keys();

  return 0;
}
